package edu.sdsc.sirius.util;

import java.io.*;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.zip.DataFormatException;
import java.util.zip.Deflater;
import java.util.zip.Inflater;

import edu.sdsc.mbt.*;
import edu.sdsc.mbt.util.*;
import edu.sdsc.mbt.viewables.*;
import edu.sdsc.sirius.io.*;
import edu.sdsc.sirius.viewers.*;
import edu.sdsc.sirius.viewers.SequenceViewerImpl.*;

public class IOHandler {
	
	private static Hashtable passComponents;
	private static String name;
	
	public static boolean selection = false;
	
	private static String[] chainIdList = new String[]{ "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", 
			"N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z" };


	/**
	 * sequences is a Vector of Sequence objects that need to be saved
	 * @param sequences
	 * @param parent
	 */
	public static void saveAlignment(Vector sequences, Manager parent){
		
		//get the desired file name
		JFileChooser chooser = new JFileChooser(parent.getLastUsedDirectory());
		chooser.setDialogTitle("Sauvegarde des squences");

		chooser.setDialogType(JFileChooser.SAVE_DIALOG);

		
		chooser.addChoosableFileFilter( new FileNameFilter("msf", "format MSF") );
		chooser.addChoosableFileFilter( new FileNameFilter("aln", "format CLUSTALW") );
		chooser.addChoosableFileFilter( new FileNameFilter("fst", "format fasta") );
		
		chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		int result = chooser.showSaveDialog(parent.getApplicationFrame());
		if (result == JFileChooser.CANCEL_OPTION){
			return;
		}
		File filename = chooser.getSelectedFile();
		String fname = filename.toString();
		
		int dot = fname.lastIndexOf(".");

		//get the desired file type
		FileFilter filter = chooser.getFileFilter();
		
		
		if (dot <= 0){
			//no extension supplied
			fname += "." + filter.toString();
		}
		
		//if typed name doesn't match selected format, go with the typed/formed string anyway
		//respect user's judgement :)

		File f = new File(fname);
		if (f.exists()){
			if (f.isDirectory()){
				parent.displayErrorMessage("slectionner un dossier de destination");

			}
			if (!f.canWrite()){
				parent.displayErrorMessage("Le fichier slectionn ne peut tre crit");
			}
			else{
				//ask the user if it's ok to overwrite the file
				if (JOptionPane.showConfirmDialog(
						parent.getApplicationFrame(), "Un fichier du mme nom existe dj. Le remplacer ?", "Confirmation",
						JOptionPane.YES_NO_OPTION) == JOptionPane.NO_OPTION){
					return;
				}
			}
		}
		
		//output file does not exist. check if the directory is writable
		String path = f.getPath();
		String fsep = File.separator;
		int separatorIndex = path.lastIndexOf(fsep);
		if (separatorIndex < 1){
			//entered file is in the current directory
			File currentDir = new File(".");
			if (!currentDir.canWrite()){
				parent.displayErrorMessage("Le rpertoire courant est en lecture seule");
			}
		}
		else{
			//entered file is in another directory
			String out_dir = path.substring(0,separatorIndex);
			File outDir = new File(out_dir);
			if (!outDir.canWrite()){
				parent.displayErrorMessage("Le rpertoire de sortie est en lecture seule");
			}
		}
		
		//save the used directory
		String dir = f.getParentFile().toString();
		parent.setLastUsedDirectory(dir);

		//set up output stream
		try{
			
			if (filter.toString().equals("fst")){
				
				FileWriter fr = new FileWriter(fname);
				PrintWriter printer = new PrintWriter(fr);

				for (int i = 0; i < sequences.size(); i++){
					Cell[] cells = (Cell[])sequences.get(i);
					Sequence s = cells[0].sequence;
					printer.println(">" + s.getName());
					
					StringBuffer buffer = new StringBuffer();
					int counter = 0;
					for (int j = 0; j < cells.length; j++){
						if (cells[j] == null) continue;
						buffer.append(cells[j].symbol);
						counter++;
						if (counter == 60){
							buffer.append('\n');
							counter = 0;
						}
					}
					printer.println(buffer);
				}
				
				
				printer.close();
				
			}
			else if (filter.toString().equals("aln")){
				FileWriter fr = new FileWriter(fname);
				PrintWriter printer = new PrintWriter(fr);
				
				//get the vector of Sequence objects
				Vector seqs = new Vector();
				for (int i = 0; i < sequences.size(); i++){
					seqs.add(((Cell[])sequences.get(i))[0].sequence);
				}
				
				printer.println("CLUSTAL\n");
				int offset = 0;
				String block;
				int span;//length of the chunk to cut out
				boolean lastPass = false;
				while (!(lastPass)){//start cycling through 60-character blocks
					
					for (int i = 0; i < sequences.size(); i++){
						
						Sequence sequence = (Sequence)seqs.get(i);
						String name = sequence.getName();
						String s = sequence.getSequence();
						
						printer.print(StringUtils.rightPadded(name, 16, " "));
						String testBlock = s.substring(offset);

						if (testBlock.length() <= 60){
							span = testBlock.length();
							lastPass = true;
						}
						else{
							span = 60;
						}
						block = s.substring(offset, offset + span);
						printer.println(block);
						
					}
					printer.println("");
					offset += 60;

				}
				printer.close();

			}
			else if (filter.toString().equals("msf")){
				FileWriter fr = new FileWriter(fname);
				PrintWriter printer = new PrintWriter(fr);
				
				//get the length
				int[] lengths = new int[sequences.size()];
				int max = 0;
				
				//get the vector of Sequence objects
				Vector seqs = new Vector();
				for (int i = 0; i < sequences.size(); i++){
					Cell[] cells = (Cell[])sequences.get(i);
					lengths[i] = cells.length;
					if (cells.length > max) max = cells.length;
					seqs.add(cells[0].sequence);
				}
				
				
				printer.println("!!AA_MULTIPLE_ALIGNMENT 1.0\n");
				printer.print(" MSF:");
				printer.print(StringUtils.leftPadded((new Integer(max)).toString(), 5, ' '));
				printer.println("  Type: P  ..");
				printer.println("");
				
				//record sequence names
				for (int i = 0; i < seqs.size(); i++){
					Sequence s = (Sequence)seqs.get(i);
					printer.print(" Name: " + s.getName() + " Len:" + StringUtils.leftPadded((new Integer(lengths[i])).toString(), 5, ' '));
					printer.println(" Check: 1 Weight: 1");
				}
				
				printer.println("\n//\n\n");
				
				
				int offset = 0;
				String block;
				int span;//length of the chunk to cut out
				boolean lastPass = false;
				while (!(lastPass)){//start cycling through 60-character blocks
					
					for (int i = 0; i < sequences.size(); i++){
						
						Sequence sequence = (Sequence)seqs.get(i);
						String name = sequence.getName();
						String s = sequence.getSequence();
						
						printer.print(StringUtils.rightPadded(name, 16, " "));
						String testBlock = s.substring(offset);

						if (testBlock.length() <= 50){
							span = testBlock.length();
							lastPass = true;
						}
						else{
							span = 50;
						}
						block = s.substring(offset, offset + span);
						block = block.replace('-', '.');
						
						//every ten characters add a space
						StringBuffer buffer = new StringBuffer();
						while(block.length() > 10){
							buffer.append(block.substring(0, 10));
							block = block.substring(10);
							buffer.append(" ");
						}
						buffer.append(block);
						
						printer.println(buffer.toString());
						
					}
					printer.println("");
					offset += 50;

				}
				printer.close();

			}
			
			
		}
		catch (Exception e){
			e.printStackTrace();
			parent.displayErrorMessage("Impossible de slectionner la structure");
			(new File(fname)).delete();
		}
		
	}
	 
	
	public static void saveTextFile(String content, Manager parent, String extension){
		
		if (content == null || content.length() == 0) return;
		
		//get the desired file name
		JFileChooser chooser = new JFileChooser(parent.getLastUsedDirectory());
		chooser.setDialogTitle("Enregistrer la structure");

		chooser.setDialogType(JFileChooser.SAVE_DIALOG);

		if (extension == null){
			chooser.addChoosableFileFilter( new FileNameFilter("txt", "format TXT") );
		}
		else{
			chooser.addChoosableFileFilter( new FileNameFilter(extension, extension + " format") );
		}
		
		chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		int result = chooser.showSaveDialog(parent.getApplicationFrame());
		if (result == JFileChooser.CANCEL_OPTION){
			return;
		}
		File filename = chooser.getSelectedFile();
		String fname = filename.toString();
		
		int dot = fname.lastIndexOf(".");

		//get the desired file type
		FileFilter filter = chooser.getFileFilter();
		
		
		if (dot <= 0){
			//no extension supplied
			fname += "." + filter.toString();
		}
		
		//if typed name doesn't match selected format, go with the typed/formed string anyway
		//respect user's judgement :)
			
		
		
		File f = new File(fname);
		if (f.exists()){
			if (f.isDirectory()){
				parent.displayErrorMessage("slectionner un dossier de destination");

			}
			if (!f.canWrite()){
				parent.displayErrorMessage("Le fichier slectionn ne peut tre crit");
			}
			else{
				//ask the user if it's ok to overwrite the file
				if (JOptionPane.showConfirmDialog(
						parent.getApplicationFrame(), "Un fichier du mme nom existe dj. Le remplacer ?", "Confirmation",
						JOptionPane.YES_NO_OPTION) == JOptionPane.NO_OPTION){
					return;
				}
			}
		}
		
		//output file does not exist. check if the directory is writable
		String path = f.getPath();
		String fsep = File.separator;
		int separatorIndex = path.lastIndexOf(fsep);
		if (separatorIndex < 1){
			//entered file is in the current directory
			File currentDir = new File(".");
			if (!currentDir.canWrite()){
				parent.displayErrorMessage("Le rpertoire courant est en lecture seule");
			}
		}
		else{
			//entered file is in another directory
			String out_dir = path.substring(0,separatorIndex);
			File outDir = new File(out_dir);
			if (!outDir.canWrite()){
				parent.displayErrorMessage("Le rpertoire de sortie est en lecture seule");
			}
		}
		
		try{
			FileWriter fr = new FileWriter(fname);
			PrintWriter printer = new PrintWriter(fr);
			printer.print(content);
			printer.close();
			
		}
		catch (Exception e){
			parent.displayExceptionMessage("Erreur  la sauvegarde du fichier texte", e);
		}

	
	}
	
	/**
	 * This method saves the provided content to a text file with the given path
	 * @param content - content of the file
	 * @param name - full path to the file
	 */
	public static void saveTextFile(String content, String name) throws IOException {
		
		if (content == null || content.length() == 0) return;
		
		FileWriter fr = new FileWriter(name);
		PrintWriter printer = new PrintWriter(fr);
		printer.print(content);
		printer.close();
			
	}
	
	public static void saveFile(Manager parent, String content, String extension, String descriptor) throws IOException {
		
		if (content == null || content.length() == 0) return;
		
		//get the desired file name
		JFileChooser chooser = new JFileChooser(parent.getLastUsedDirectory());
		chooser.setDialogTitle("Sauvegarde des donnes");

		chooser.setDialogType(JFileChooser.SAVE_DIALOG);
		
		FileFilter filter = new FileNameFilter(extension, descriptor);
		chooser.setFileFilter(filter);
		
		chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		int result = chooser.showSaveDialog(parent.getApplicationFrame());
		if (result == JFileChooser.CANCEL_OPTION){
			return;
		}
		File filename = chooser.getSelectedFile();
		String fname = filename.toString();
		
		int dot = fname.lastIndexOf(".");

		//get the desired file type
		filter = chooser.getFileFilter();
		
		if (dot <= 0){
			//no extension supplied
			fname += "." + filter.toString();
		}
		
		File f = new File(fname);
		if (f.exists()){
			if (f.isDirectory()){
				parent.displayErrorMessage("slectionner un dossier de destination");
				return;

			}
			if (!f.canWrite()){
				parent.displayErrorMessage("Le fichier slectionn ne peut tre crit");
				return;
			}
		}
		
		//use f to write the content
		FileWriter fr = new FileWriter(f);
		PrintWriter printer = new PrintWriter(fr);
		printer.print(content);
		printer.close();
		
		
	}
		
	public static void saveStructureFile(StructureEntry structureEntry, Manager parent){
		
		Structure structure = structureEntry.getStructure();
		
		if (selection){
			if (structure.getStructureMap().getStructureStyles().getSelectionFlag() == StructureStyles.FLAG_NONE){
				parent.displayErrorMessage("Il n\'y a rien de slectionn dans la structure");
				return;
			}
		}
		
		HashMap transformed = null;
		
		//determine whether the coordinates had been transformed
		if (parent.getStructureViewer().getInitialCoordinates(structure) != null){
			//there was transformation
			
			if (JOptionPane.showConfirmDialog(
					parent.getApplicationFrame(), "La structure a t modifie. Sauvegarder les nouvelles coordonnes ?", "Confirmation",
					JOptionPane.YES_NO_OPTION) == JOptionPane.NO_OPTION){
				transformed = parent.getStructureViewer().getInitialCoordinates(structure);
			}

		}
		
		//get the desired file name
		JFileChooser chooser = new JFileChooser(parent.getLastUsedDirectory());
		chooser.setDialogTitle("Enregistrer la structure");

		chooser.setDialogType(JFileChooser.SAVE_DIALOG);

		
		FileFilter pdb = new FileNameFilter("pdb", "format PDB");
		
		chooser.addChoosableFileFilter( pdb );
		chooser.addChoosableFileFilter( new FileNameFilter("mol2", "format MOL2") );
//		chooser.addChoosableFileFilter( new FileNameFilter("xml", "GEMSTONE XML format") );
		chooser.addChoosableFileFilter( new FileNameFilter("sdf", "format SDF"));
		chooser.addChoosableFileFilter( new FileNameFilter("inp", "GAMESS coordinate format") );
		
		chooser.setFileFilter(pdb);
		
		chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		int result = chooser.showSaveDialog(parent.getApplicationFrame());
		if (result == JFileChooser.CANCEL_OPTION){
			return;
		}
		File filename = chooser.getSelectedFile();
		String fname = filename.toString();
		
		int dot = fname.lastIndexOf(".");

		//get the desired file type
		FileFilter filter = chooser.getFileFilter();
		
		
		if (dot <= 0){
			//no extension supplied
			fname += "." + filter.toString();
		}
		
//		System.out.println("fname = " + fname);
		
		//if typed name doesn't match selected format, go with the typed/formed string anyway
		//respect user's judgement :)
		
		boolean rename = true;
			
		//check whether the structure contains any amino acids or nucleic acids (is a protein or a dna)
		//if not, check whether there are any atoms with identical names. If so, run renaming
		//of atoms
		for (int i = 0; i < structure.getStructureMap().getResidueCount(); i++){
			Residue r = structure.getStructureMap().getResidue(i);
			if (ProteinProperties.isAminoAcid(r.getCompoundCode()) || DNAProperties.isNucleicAcid(r.getCompoundCode())){
				rename = false;
				break;//there is at least one amino acid, leave it alone
			}
		}
		
		//if rename is still true, it means that the structure is not a protein or dna
		//check whether there are identical atom names
		if (rename){
			boolean identical = false;
			HashMap names = new HashMap();
			for (int i = 0; i < structure.getStructureMap().getAtomCount(); i++){
				Atom a = structure.getStructureMap().getAtom(i);
				if (names.containsKey(a.name)){
					identical = true;
				}
				else{
					names.put(a.name, null);
				}
			}
			
			if (!identical) rename = false;//if all atoms have different names, do not rename
		}
		
		File f = new File(fname);
		if (f.exists()){
			if (f.isDirectory()){
				parent.displayErrorMessage("slectionner un dossier de destination");

			}
			if (!f.canWrite()){
				parent.displayErrorMessage("Le fichier slectionn ne peut tre crit");
			}
			else{
				//check whether it's an sdf file
				if (fname.endsWith("sdf") || fname.endsWith("SDF")){
					if (structure.getStructureMap().getAtomCount() > 999){
						parent.displayErrorMessage("Le format SDF ne supporte actuellement que les structures contenant moins de 1000 atomes\n" +
								"             Choisir un autre format pour viter une perte de donnes");
						return;
						
					}
					int reply = JOptionPane.showConfirmDialog(
							parent.getApplicationFrame(), "Un fichier de ce nom existe dj. Ajouter le nouveau fichier  la fin ?\n" +
							"Si vous slectionnez \"No\" le fichier sera remplac", "Confirmation",
							JOptionPane.YES_NO_CANCEL_OPTION);
					if (reply == JOptionPane.NO_OPTION){
						//overwrite
						String content = convertStructureToSDF(structureEntry, parent.getStructureViewer().getStructureName(structure), transformed);
						try{
							FileWriter fr = new FileWriter(fname);
							PrintWriter printer = new PrintWriter(fr);
							
							printer.println(content);
							printer.flush();
							printer.close();
						}
						catch (Exception e){
							parent.displayExceptionMessage("Erreur  la sauvegarde d\'un fichier", e);
						}
						return;
					}
					else if (reply == JOptionPane.YES_OPTION){
						//append
						try {
							String content = convertStructureToSDF(structureEntry, parent.getStructureViewer().getStructureName(structure), transformed);
							BufferedWriter out = new BufferedWriter(new FileWriter(fname, true));
							out.write(content);
							out.close();
						}
						catch (IOException e) {
							parent.displayExceptionMessage("Erreur  la sauvegarde d\'un fichier", e);
						}

						
						return;
					}
					else{
						//cancel - do nothing
						return;
					}
				}
				else{
					//ask the user if it's ok to overwrite the file
					if (JOptionPane.showConfirmDialog(
							parent.getApplicationFrame(), "Un fichier du mme nom existe dj. Le remplacer ?", "Confirmation",
							JOptionPane.YES_NO_OPTION) == JOptionPane.NO_OPTION){
						return;
					}
				}
			}
		}
		
		//output file does not exist. check if the directory is writable
		String path = f.getPath();
		String fsep = File.separator;
		int separatorIndex = path.lastIndexOf(fsep);
		if (separatorIndex < 1){
			//entered file is in the current directory
			File currentDir = new File(".");
			if (!currentDir.canWrite()){
				parent.displayErrorMessage("Le rpertoire courant est en lecture seule");
			}
		}
		else{
			//entered file is in another directory
			String out_dir = path.substring(0,separatorIndex);
			File outDir = new File(out_dir);
			if (!outDir.canWrite()){
				parent.displayErrorMessage("Le rpertoire de sortie est en lecture seule");
			}
		}
		
		//save the used directory
		String dir = f.getParentFile().toString();
		parent.setLastUsedDirectory(dir);

		//set up output stream
		try{
			
			if (filter.toString().equals("pdb")){
				savePdb(structure, fname, parent, transformed, rename, false);
			}
			else if (filter.toString().equals("mol2")){
				saveMol2(structure, fname, parent, transformed, rename);
			}
			else if (filter.toString().equals("inp")){
				saveInp(structure, fname, transformed);
			}
			else if (filter.toString().equals("sdf")){
				saveSDF(structureEntry, fname, parent.getStructureViewer().getStructureName(structure), transformed);
			}
		}
		catch (Exception e){
			e.printStackTrace();
			parent.displayErrorMessage("Impossible de slectionner la structure");
			(new File(fname)).delete();
		}
		
		
	}
	
	
	private static void saveInp(Structure structure, String fname, HashMap initial) throws IOException {
		
		FileWriter fr = new FileWriter(fname);
		PrintWriter printer = new PrintWriter(fr);
		
		//get the styles in case selection only is required
		StructureStyles styles = structure.getStructureMap().getStructureStyles();
		
		printer.println("$DATA");
		printer.println(structure.getUrlString());
		printer.println("C1");
		
		for (int i = 0; i < structure.getStructureMap().getAtomCount(); i++){
			Atom a = structure.getStructureMap().getAtom(i);
			if (selection){
				if (!structure.getStructureMap().getStructureStyles().isSelected(a))continue;
			}
			
			printer.print(StringUtils.rightPadded(a.getElement(),12," "));
			printer.print(StringUtils.rightPadded((new Integer(a.element)).toString(), 5, " "));
			if (initial != null){
				printer.print(StringUtils.rightPadded((new Double(((double[])initial.get(a))[0])).toString(), 13, "0"));
				printer.print(" ");
				printer.print(StringUtils.rightPadded((new Double(((double[])initial.get(a))[1])).toString(), 13, "0"));
				printer.print(" ");
				printer.println(StringUtils.rightPadded((new Double(((double[])initial.get(a))[2])).toString(), 13, "0"));
			}
			else{
				printer.print(StringUtils.rightPadded((new Double(a.coordinate[0])).toString(), 13, "0"));
				printer.print(" ");
				printer.print(StringUtils.rightPadded((new Double(a.coordinate[1])).toString(), 13, "0"));
				printer.print(" ");
				printer.println(StringUtils.rightPadded((new Double(a.coordinate[2])).toString(), 13, "0"));
			}
			
		}
		
		printer.println("$END");
		printer.flush();
		printer.close();
	}
	
	public static void saveSDF(StructureEntry structureEntry, String fname, String name, HashMap initial) throws IOException {
		
		String content = convertStructureToSDF(structureEntry, name, initial);
		FileWriter fr = new FileWriter(fname);
		PrintWriter printer = new PrintWriter(fr);
		printer.print(content);
		printer.flush();
		printer.close();
		
	}
	
	public static String convertStructureToSDF(StructureEntry structureEntry, String name, HashMap initial){
		return convertStructureToSDF(structureEntry.getStructure(), name, initial, structureEntry.getMetadata());
	}
	
	public static String convertStructureToSDF(Structure structure, String name, HashMap initial, MetaData metadata){
		
		StructureMap structureMap = structure.getStructureMap();
		
		StringBuffer out = new StringBuffer();
		out.append(name);
		out.append("\n\n\n");
		
		//add the counts line
		int atoms = structure.getStructureMap().getAtomCount();
		int bonds = structure.getStructureMap().getBondCount();
		
		String a = StringUtils.leftPadded((new Integer(atoms)).toString(), 3, ' ');
		String b = StringUtils.leftPadded((new Integer(bonds)).toString(), 3, ' ');
		out.append(a);
		out.append(b);
		out.append("  0     0  0  0  0  0  0    V2000\n");
		
		HashMap map = new HashMap();
				
		//take care of the atomic coordinate block
		for (int i = 0; i < structureMap.getAtomCount(); i++){
			Atom atom = structureMap.getAtom(i);
			if (selection){
				if (!structureMap.getStructureStyles().isSelected(atom)) continue;
			}
			
			double x = 0;
			double y = 0;
			double z = 0;

			if (initial == null){
				//save transformed coordinates
				x = atom.coordinate[0];
				y = atom.coordinate[1];
				z = atom.coordinate[2];
			}
			else{
				x = ((double[])initial.get(atom))[0];
				y = ((double[])initial.get(atom))[1];
				z = ((double[])initial.get(atom))[2];
			}
			
			String xs = StringUtils.getFormattedNumber(x, 4);
			String ys = StringUtils.getFormattedNumber(y, 4);
			String zs = StringUtils.getFormattedNumber(z, 4);
			
			out.append(StringUtils.leftPadded(xs, 10, ' '));
			out.append(StringUtils.leftPadded(ys, 10, ' '));
			out.append(StringUtils.leftPadded(zs, 10, ' '));
			
			out.append(" ");
			out.append(StringUtils.rightPadded(atom.getElement(), 3, ' '));
			out.append(" 0");
			
			int charge = 0;
			if (atom.partialCharge == 1.0f){
				charge = 3;
			}
			else if (atom.partialCharge == 2.0f){
				charge = 2;
			}
			else if (atom.partialCharge == 3.0f){
				charge = 1;
			}
			else if (atom.partialCharge == -1.0f){
				charge = 5;
			}
			else if (atom.partialCharge == -2.0f){
				charge = 6;
			}
			else if (atom.partialCharge == -3.0f){
				charge = 7;
			}
			
			out.append(StringUtils.leftPadded((new Integer(charge)).toString(), 3, ' '));
			out.append("  0  0  0  0  0  0  0  0  0\n");
			
			int j = i + 1;
			
			map.put(atom, new Integer(j));
		}
		
		//do the bonds
		for (int i = 0; i < structureMap.getBondCount(); i++){
			Bond bond = structureMap.getBond(i);
			if (selection){
				if (!structureMap.getStructureStyles().isSelected(bond.getAtom(0)) || !structureMap.getStructureStyles().isSelected(bond.getAtom(1))){
					continue;
				}
			}
			
			int a1 = ((Integer)map.get(bond.getAtom(0))).intValue();
			int a2 = ((Integer)map.get(bond.getAtom(1))).intValue();
			float n = bond.order;
			
			int order = 1;
			if (n == 1.5){
				order = 4;
			}
			else if (n == 2){
				order = 2;
			}
			else if (n == 3){
				order = 3;
			}
			
			String aa1 = StringUtils.leftPadded((new Integer(a1)).toString(), 3, ' ');
			String aa2 = StringUtils.leftPadded((new Integer(a2)).toString(), 3, ' ');
			String oo = StringUtils.leftPadded((new Integer(order)).toString(), 3, ' ');
			
			out.append(aa1);
			out.append(aa2);
			out.append(oo);
			out.append("  0  0  0  0\n");
			
		}
		
		out.append("M  END\n");
		
		//check for score
		if (metadata != null){
			if (metadata != null && metadata.score < 1000000){
				out.append(">  <SCORE>\n");
				out.append(StringUtils.getFormattedNumber(metadata.score, 3));
				out.append("\n \n");
			}
			out.append("$$$$\n");
		}
		else{
			out.append("$$$$\n");
		}
		return out.toString();
		
		
	}
	
	public static String convertStructureToPdb(Structure structure, Manager parent, boolean includeHydrogens){
		
		StringBuffer out = new StringBuffer();
		//get the styles in case selection only is required
		StructureStyles styles = structure.getStructureMap().getStructureStyles();
		
		StructureViewer viewer = parent.getStructureViewer();
		HashMap transformed = null;
		if (viewer != null && viewer.getTransformedCoordinates().size() > 0){
			transformed = viewer.getTransformedCoordinates();
		}


		Vector heteroatoms = new Vector();
		//iterate over all atoms in the structure
		int counter = 1;//atom number counter: will use this, in case reshuffling will be needed (e.g., with added hydrogens in proteins)
		HashMap reference = new HashMap();//old atom number -> new counter
		for (int i = 0 ; i < structure.getStructureMap().getResidueCount(); i++){
			
			Residue residue = structure.getStructureMap().getResidue(i);
			//walk through the atoms of this residue and save them in order
			for (int j = 0; j < residue.getAtomCount(); j++){
				Atom a = residue.getAtom(j);
				if (selection){
					if (!styles.isSelected(a))continue;
				}
				
				if (!includeHydrogens && a.element == 1) continue;
				
				reference.put(new Integer(a.number), new Integer(counter));
				
				if (ProteinProperties.isAminoAcid(a.compound) || DNAProperties.isNucleicAcid(a.compound)){
					//the ATOM keyword
					out.append(StringUtils.rightPadded("ATOM",6," "));
				}
				else{
					out.append(StringUtils.rightPadded("HETATM",6," "));
					heteroatoms.add(a);
				}
				
				//atom number
				if (PeriodicTable.getElementSymbol(a.element).length() > 1){
					//start the name immediately
					out.append(StringUtils.leftPadded((new Integer(counter)).toString(),5, " ") + " ");
				
					//atom type
					out.append(StringUtils.rightPadded(a.name, 4, " ") + " ");
				}
				else{
					out.append(StringUtils.leftPadded((new Integer(counter)).toString(),5, " ") + "  ");
					
						//atom type
					out.append(StringUtils.rightPadded(a.name, 3, " ") + " ");
					
				}
				
				//residue
				out.append(StringUtils.rightPadded(a.compound, 3, " ") + " ");
				
				//chain
				if (a.chain_id.equals("_")){
					out.append(" ");
				}
				else{
					out.append(StringUtils.rightPadded(a.chain_id, 1, " "));
				}
				
				//residue number
				out.append(StringUtils.leftPadded((new Integer(a.residue_id)).toString(), 4, " ") + "     ");
				
				//coordinates
				String x = null;
				String y = null;
				String z = null;
				
				if (transformed != null && transformed.containsKey(a)){
					x = (new Double(((double[])transformed.get(a))[0])).toString();
					y = (new Double(((double[])transformed.get(a))[1])).toString();
					z = (new Double(((double[])transformed.get(a))[2])).toString();
				}
				else{
					x = (new Double(a.coordinate[0])).toString();
					y = (new Double(a.coordinate[1])).toString();
					z = (new Double(a.coordinate[2])).toString();
				}
				
				//make sure the part of the number after the point has three digits
				int index = x.indexOf(".");
				String subX = x.substring(0, index+1);
				String ext = x.substring(index+1);
				x = subX + StringUtils.rightPadded(ext, 3, "0");
				
				index = y.indexOf(".");
				String subY = y.substring(0, index+1);
				ext = y.substring(index+1);
				y = subY + StringUtils.rightPadded(ext, 3, "0");
	
				index = z.indexOf(".");
				String subZ = z.substring(0, index+1);
				ext = z.substring(index+1);
				z = subZ + StringUtils.rightPadded(ext, 3, "0");
	
				out.append(StringUtils.leftPadded(x, 7, " ") + " ");
				out.append(StringUtils.leftPadded(y, 7, " ") + " ");
				out.append(StringUtils.leftPadded(z, 7, " ") + "  ");
				
				//occupancy
				out.append(StringUtils.rightPadded("1.00", 4, " "));
				
				//bfactor
				out.append(StringUtils.leftPadded("0.00", 6, " "));
				
				//the spacing
				out.append(StringUtils.leftPadded(" ", 11, " "));
				
				out.append(StringUtils.rightPadded(a.getElement(), 3, " "));
				
				out.append("\n");
				counter++;
			}

		}
		
		if (heteroatoms.size() > 0){
			//generate conect records
			for (int i = 0; i < heteroatoms.size(); i++){
				Atom a = (Atom)heteroatoms.get(i);
				Vector bonds = a.structure.getStructureMap().getBonds(a);
				if (bonds == null) continue;
				Vector numbers = new Vector();//neighbor numbers to be printed
				//walk through the bonds to determine bond order and to sort them
				for (int j = 0; j < bonds.size(); j++){
					Bond b = (Bond)bonds.get(j);
					int order = (int)b.order;
					if (b.getAtom(0) == a){
						for (int k = 0; k < order; k++){
							if (selection){
								if (styles.isSelected(b.getAtom(1))){
									numbers.add(reference.get(new Integer(b.getAtom(1).number)));
								}
							}
							else if (!includeHydrogens){
								if (b.getAtom(1).element != 1){
									numbers.add(reference.get(new Integer(b.getAtom(1).number)));
								}
							}
							else{
								numbers.add(reference.get(new Integer(b.getAtom(1).number)));
							}
						}
					}
					else if (b.getAtom(1) == a){
						for (int k = 0; k < order; k++){
							if (selection){
								if (styles.isSelected(b.getAtom(0))){
									numbers.add(reference.get(new Integer(b.getAtom(0).number)));
								}
							}
							else if (!includeHydrogens){
								if (b.getAtom(0).element != 1){
									numbers.add(reference.get(new Integer(b.getAtom(0).number)));
								}
							}
							else{
								numbers.add(reference.get(new Integer(b.getAtom(0).number)));
							}
						}
					}
				}

				Collections.sort(numbers);
			
				out.append(StringUtils.rightPadded("CONECT",6," "));
				out.append(StringUtils.leftPadded((reference.get(new Integer(a.number))).toString(), 5, " "));
				for (int j = 0; j < numbers.size(); j++){
					out.append(StringUtils.leftPadded(numbers.get(j).toString(), 5, " "));
				}
				out.append("\n");
			}
			
		}
		out.append("END\n");
		
		
		return out.toString();
	}
	
	
	public static void savePdb(Structure structure, String fname, Manager parent, HashMap initial, boolean rename, boolean skipIons) throws IOException {
		
		FileWriter fr = new FileWriter(fname);
		PrintWriter printer = new PrintWriter(fr);
		
//		System.out.println("rename = " + rename);
		
		//get the styles in case selection only is required
		StructureStyles styles = structure.getStructureMap().getStructureStyles();
		
		StructureViewer viewer = parent.getStructureViewer();
		HashMap transformed = null;
		if (viewer != null && viewer.getTransformedCoordinates().size() > 0){
			transformed = viewer.getTransformedCoordinates();
		}
		
		int chainIdCounter = 0;

		HashMap chainReplacement = new HashMap();//in case long chain ids need to be replaced with single letters
		for (int i = 0; i < structure.getStructureMap().getChainCount(); i++){
			Chain c = structure.getStructureMap().getChain(i);
			if (c.getChainId().length() > 1){
				chainReplacement.put(c.getChainId(), IOHandler.chainIdList[chainIdCounter++]);
			}
		}
		
		Vector heteroatoms = new Vector();
		//iterate over all atoms in the structure
		int counter = 1;//atom number counter: will use this, in case reshuffling will be needed (e.g., with added hydrogens in proteins)
		HashMap reference = new HashMap();//old atom number -> new counter
		
		HashMap atomNames = new HashMap();//element -> current index
		for (int i = 0 ; i < structure.getStructureMap().getResidueCount(); i++){
			
			Residue residue = structure.getStructureMap().getResidue(i);
			//walk through the atoms of this residue and save them in order
			for (int j = 0; j < residue.getAtomCount(); j++){
				Atom a = residue.getAtom(j);
				
				//check whether this is a Na+ ion - added for exporting PDB file in a MM-PBSA energy calculations
				if (skipIons && a.element == 11) continue;
				
				if (rename){
					try{
						int index = (Integer)atomNames.get(a.element);
						a.name = PeriodicTable.getElementSymbol(a.element) + index;
						atomNames.put(a.element, ++index);
					}
					catch (NullPointerException ex){
						//no index was entered for this atom
						a.name = PeriodicTable.getElementSymbol(a.element) + 1;
						atomNames.put(a.element, 2);
					}
					
					//rename also means that all atoms must go into the same residue
					//perhaps named UNK
					a.residue_id = 1;
					a.residue.setCompoundCode("UNK");
					a.compound = "UNK";
				}
				
				
				if (selection){
					if (!styles.isSelected(a))continue;
				}
				
				reference.put(new Integer(a.number), new Integer(counter));
				
				if (ProteinProperties.isAminoAcid(a.compound) || DNAProperties.isNucleicAcid(a.compound)){
					//the ATOM keyword
					printer.print(StringUtils.rightPadded("ATOM",6," "));
				}
				else{
					printer.print(StringUtils.rightPadded("HETATM",6," "));
					heteroatoms.add(a);
				}
				
				//atom number
				if (PeriodicTable.getElementSymbol(a.element).length() > 1){
					//start the name immediately
					printer.print(StringUtils.leftPadded((new Integer(counter)).toString(),5, " ") + " ");
				
					//atom type
					printer.print(StringUtils.rightPadded(a.name, 4, " ") + " ");
				}
				else{
					if (a.name.length() > 3){
						printer.print(StringUtils.leftPadded((new Integer(counter)).toString(),5, " ") + " ");
						
						//atom type
						printer.print(StringUtils.rightPadded(a.name, 4, " ") + " ");
					}
					else{
						printer.print(StringUtils.leftPadded((new Integer(counter)).toString(),5, " ") + "  ");
						
						//atom type
						printer.print(StringUtils.rightPadded(a.name, 3, " ") + " ");
					}
					
				}
				
				//residue
				printer.print(StringUtils.rightPadded(a.compound, 3, " ") + " ");
				
				//chain
				if (a.chain_id.equals("_")){
					printer.print(" ");
				}
				else{
					if (a.chain_id.length() > 1){
						printer.print(StringUtils.rightPadded((String)chainReplacement.get(a.chain_id), 1, " "));
					}
					else{
						printer.print(StringUtils.rightPadded(a.chain_id, 1, " "));
					}
				}
				
				//residue number
				printer.print(StringUtils.leftPadded((new Integer(a.residue_id)).toString(), 4, " ") + "     ");
//				System.out.println("residue id = " + a.residue_id);
				
				//coordinates
				String x = null;
				String y = null;
				String z = null;
				if (initial != null){
					x = StringUtils.getFormattedNumber(((double[])initial.get(a))[0], 3);
					y = StringUtils.getFormattedNumber(((double[])initial.get(a))[1], 3);
					z = StringUtils.getFormattedNumber(((double[])initial.get(a))[2], 3);
				}
				else{
					if (transformed != null && transformed.containsKey(a)){
						x = StringUtils.getFormattedNumber(((double[])transformed.get(a))[0], 3);
						y = StringUtils.getFormattedNumber(((double[])transformed.get(a))[1], 3);
						z = StringUtils.getFormattedNumber(((double[])transformed.get(a))[2], 3);
					}
					else{
						x = StringUtils.getFormattedNumber(a.coordinate[0], 3);
						y = StringUtils.getFormattedNumber(a.coordinate[1], 3);
						z = StringUtils.getFormattedNumber(a.coordinate[2], 3);
					}
				}
				
				//make sure the part of the number after the point has three digits
				int index = x.indexOf(".");
				String subX = x.substring(0, index+1);
				String ext = x.substring(index+1);
				x = subX + StringUtils.rightPadded(ext, 3, "0");
				
				index = y.indexOf(".");
				String subY = y.substring(0, index+1);
				ext = y.substring(index+1);
				y = subY + StringUtils.rightPadded(ext, 3, "0");
	
				index = z.indexOf(".");
				String subZ = z.substring(0, index+1);
				ext = z.substring(index+1);
				z = subZ + StringUtils.rightPadded(ext, 3, "0");
	
				printer.print(StringUtils.leftPadded(x, 7, " ") + " ");
				printer.print(StringUtils.leftPadded(y, 7, " ") + " ");
				printer.print(StringUtils.leftPadded(z, 7, " ") + "  ");
				
				//occupancy
				printer.print(StringUtils.rightPadded("1.00", 4, " "));
				
				//bfactor
				printer.print(StringUtils.leftPadded("0.00", 6, " "));
				
				//the spacing
				printer.print(StringUtils.leftPadded(" ", 11, " "));
				
				printer.print(StringUtils.rightPadded(a.getElement(), 3, " "));
				
				printer.print("\n");
				counter++;
			}

		}
		
		if (heteroatoms.size() > 0){
			//generate conect records
			for (int i = 0; i < heteroatoms.size(); i++){
				Atom a = (Atom)heteroatoms.get(i);
				Vector bonds = a.structure.getStructureMap().getBonds(a);
				if (bonds == null) continue;
				Vector numbers = new Vector();//neighbor numbers to be printed
				//walk through the bonds to determine bond order and to sort them
				for (int j = 0; j < bonds.size(); j++){
					Bond b = (Bond)bonds.get(j);
					int order = (int)b.order;
					if (b.getAtom(0) == a){
						for (int k = 0; k < order; k++){
							if (selection){
								if (styles.isSelected(b.getAtom(1))){
									numbers.add(reference.get(new Integer(b.getAtom(1).number)));
								}
							}
							else{
								numbers.add(reference.get(new Integer(b.getAtom(1).number)));
							}
						}
					}
					else if (b.getAtom(1) == a){
						for (int k = 0; k < order; k++){
							if (selection){
								if (styles.isSelected(b.getAtom(0))){
									numbers.add(reference.get(new Integer(b.getAtom(0).number)));
								}
							}
							else{
								numbers.add(reference.get(new Integer(b.getAtom(0).number)));
							}
						}
					}
				}

				Collections.sort(numbers);
			
				printer.print(StringUtils.rightPadded("CONECT",6," "));
				printer.print(StringUtils.leftPadded((reference.get(new Integer(a.number))).toString(), 5, " "));
				for (int j = 0; j < numbers.size(); j++){
					printer.print(StringUtils.leftPadded(numbers.get(j).toString(), 5, " "));
				}
				printer.print("\n");
			}
			
		}
		printer.println("END");
		printer.close();

	}

	/**
	 * This method writes the structure to a file as mol2. Note that if rename flag is set to false, but it's a non-protein
	 * or dna structure, the renaming is still done. This is necessary to avoid problems with the resulting file format.
	 * @param structure
	 * @param fname
	 * @param parent
	 * @param initial
	 * @param rename
	 * @throws IOException
	 */
	public static void saveMol2(Structure structure, String fname, Manager parent, HashMap initial, boolean rename) throws IOException {
		
		//first, check if the data is with bond order info
		StructureMap structureMap = structure.getStructureMap();
		StructureStyles styles = structureMap.getStructureStyles();
		
		if (!rename){
			boolean temp = true;
			for (int i = 0; i < structure.getStructureMap().getResidueCount(); i++){
				Residue r = structure.getStructureMap().getResidue(i);
				if (ProteinProperties.isAminoAcid(r.getCompoundCode())){
					temp = false;
					break;
				}
				if (DNAProperties.isNucleicAcid(r.getCompoundCode())){
					temp = false;
					break;
				}
			}
			
			rename = temp;//if there are no protein or DNA residues, rename the atoms
		}
		
		//just in case, keep a reference map of initial atom numbers vs written ones 
		//this is necessary if atom numbers in the structure are not consecutive
		HashMap num = new HashMap();

		FileWriter fr = new FileWriter(fname);
		PrintWriter printer = new PrintWriter(fr);
		
		printer.print("#\n");
		printer.print("#\tCreating user name:\tSirius export\n");
		printer.print("#\tCreation time:\t\t" + Calendar.getInstance().getTime() + "\n\n");
		
		printer.println("@<TRIPOS>MOLECULE");
		printer.println(parent.getStructureViewer().getStructureName(structure));
		printer.print(structureMap.getAtomCount() + " " + structureMap.getBondCount());
		if (structureMap.getResidueCount() > 1){
			printer.println(" " + structureMap.getResidueCount());
		}
		else{
			printer.print("\n");
		}
		printer.print("SMALL\n");
		
		if (structureMap.chargesUserAssigned){
			printer.println("USER_CHARGES");
		}
		else{
			printer.println("****");
		}
		
		//determine whether atom numbering needs to be rolled back to 1
		int start = structureMap.getAtom(0).number;
		int factor = start - 1;
		
		StructureViewer viewer = parent.getStructureViewer();
		HashMap transformed = null;
		if (viewer != null && viewer.getTransformedCoordinates().size() > 0){
			transformed = viewer.getTransformedCoordinates();
		}

		
		//with selection option, atoms may need to be renumbered
//		HashMap reference = new HashMap();//number in original structure -> consecutive number in file
		
		int count = 1;
		
		int chainIdCounter = 0;

		HashMap chainReplacement = new HashMap();//in case long chain ids need to be replaced with single letters
		for (int i = 0; i < structure.getStructureMap().getChainCount(); i++){
			Chain c = structure.getStructureMap().getChain(i);
			if (c.getChainId().length() > 1){
				chainReplacement.put(c.getChainId(), IOHandler.chainIdList[chainIdCounter++]);
			}
		}

		
		printer.println("@<TRIPOS>ATOM");
		HashMap atomNames = new HashMap();
		
		for (int i = 0; i < structureMap.getChainCount(); i++){
			
			Chain chain = structureMap.getChain(i);
			
			for (int j = 0; j < structureMap.getResidueCount(); j++){
				Residue r = structureMap.getResidue(j);
				if (!r.getChainId().equals(chain.getChainId()))continue;
				for (int k = 0; k < r.getAtomCount(); k++){
					Atom a = r.getAtom(k);
					if (selection){
						if (!styles.isSelected(a)) continue;
					}

					if (rename){
						try{
							int index = (Integer)atomNames.get(a.element);
							a.name = PeriodicTable.getElementSymbol(a.element) + index;
							atomNames.put(a.element, ++index);
						}
						catch (NullPointerException ex){
							//no index was entered for this atom
							a.name = PeriodicTable.getElementSymbol(a.element) + 1;
							atomNames.put(a.element, 2);
						}
						
						//rename also means that all atoms must go into the same residue
						//perhaps named UNK
						a.residue_id = 1;
						a.residue.setCompoundCode("UNK");
						a.compound = "UNK";
					}

					num.put(a, new Integer(count));
					printer.print(StringUtils.leftPadded((new Integer(count)).toString(), 7, " ") + " ");
					
					printer.print(StringUtils.rightPadded(a.name, 8, " ") + " ");
				
					String x = null;
					String y = null;
					String z = null;
					
					if (initial != null){
						x = StringUtils.getFormattedNumber(((double[])initial.get(a))[0], 4);
						y = StringUtils.getFormattedNumber(((double[])initial.get(a))[0], 4);
						z = StringUtils.getFormattedNumber(((double[])initial.get(a))[0], 4);
					}
					else{
						if (transformed != null && transformed.containsKey(a)){
							x = StringUtils.getFormattedNumber(((double[])transformed.get(a))[0], 4);
							y = StringUtils.getFormattedNumber(((double[])transformed.get(a))[0], 4);
							z = StringUtils.getFormattedNumber(((double[])transformed.get(a))[0], 4);
						}
						else{
							x = StringUtils.getFormattedNumber(a.coordinate[0], 4);
							y = StringUtils.getFormattedNumber(a.coordinate[1], 4);
							z = StringUtils.getFormattedNumber(a.coordinate[2], 4);
						}
					}
					
					//make sure the part of the number after the point has four digits
					int index = x.indexOf(".");
					String subX = x.substring(0, index+1);
					String ext = x.substring(index+1);
					x = subX + StringUtils.rightPadded(ext, 4, "0");
					
					index = y.indexOf(".");
					String subY = y.substring(0, index+1);
					ext = y.substring(index+1);
					y = subY + StringUtils.rightPadded(ext, 4, "0");

					index = z.indexOf(".");
					String subZ = z.substring(0, index+1);
					ext = z.substring(index+1);
					z = subZ + StringUtils.rightPadded(ext, 4, "0");

					printer.print(StringUtils.leftPadded(x, 9, " ") + " ");
					printer.print(StringUtils.leftPadded(y, 9, " ") + " ");
					printer.print(StringUtils.leftPadded(z, 9, " ") + " ");
				
				
					//form the element type string
					if (a.element == 1){
						printer.print(StringUtils.rightPadded("H", 6, " ") + " ");
					}
					else{
						Vector b = structureMap.getBonds(a);
//						System.out.println("b = " + b + " for " + a.name);
						if (a.element == 7){
							//check for explicit hybridization record
							if (a.hybridization == Atom.SP2){
								printer.print(StringUtils.rightPadded("N.2", 6, " ") + " ");
							}
							else{
								if (a.name.equals("N") && structureMap.getResidue(a).getClassification() == Residue.COMPOUND_AMINO_ACID){
									//check for N-terminus
									Vector aa = structureMap.getAtoms(a);
									int counter = 0;
									if (aa == null || aa.size() == 0){
										printer.print(StringUtils.rightPadded("N", 6, " ") + " ");
									}
									else{
										for (int m = 0; m < aa.size(); m++){
											if (((Atom)aa.get(m)).element == 1){
												counter++;
											}
										}
										if (counter == 2){
											printer.print(StringUtils.rightPadded("N.3", 6, " ") + " ");
										}
										else{
											printer.print(StringUtils.rightPadded("N.am", 6, " ") + " ");
										}
									}
								}
								else if (b != null && b.size() == 3){
									printer.print(StringUtils.rightPadded("N.3", 6, " ") + " ");
								}
								else{
									printer.print(StringUtils.rightPadded("N", 6, " ") + " ");
								}
							}
						}
						else{
							String type = a.getElement() + ".3";
							//if hybridization different from SP3 is known, just record it
							if (a.hybridization == Atom.SP2){
								type = a.getElement() + ".2";
							}
							else{
								//check for bond orders around the atom
								if (b != null){
									for (int m = 0; m < b.size(); m++){
										if (a.element == 6 && ((Bond)b.get(m)).order == 1.5){
											type = "C.ar";
											break;
										}
										if (((Bond)b.get(m)).order == 2){
											type = a.getElement() + ".2";
											break;
										}
									}
								}
							}
							printer.print(StringUtils.rightPadded(type, 6, " ") + " ");
						}
					}
					
					printer.print(StringUtils.leftPadded((new Integer(a.residue_id)).toString(), 4, " ") + " ");
					
					printer.print(StringUtils.rightPadded(a.compound, 8, " ") + " ");
					
					String q = (new Double(a.partialCharge)).toString();
					index = q.indexOf(".");
					String subQ = q.substring(0, index+1);
					String ex = q.substring(index+1);
					q = subQ + StringUtils.rightPadded(ex, 4, "0");

					printer.print(StringUtils.leftPadded(q, 8, " ") + " ");
					
					if (structureMap.getResidueCount() > 1){
						if (a.backbone){
							printer.print("BACKBONE\n");
						}
						else{
							printer.print("****\n");
						}
					}
					else{
						printer.print("\n");
					}
					
					count++;
				}
			}
		}
		
		printer.println("@<TRIPOS>BOND");
		int counter = 1;
		for (int i = 0; i < structureMap.getBondCount(); i++){
			Bond b = structureMap.getBond(i);
			if (selection){
				if (styles.isSelected(b.getAtom(0)) && styles.isSelected(b.getAtom(1))){
					int a1 = ((Integer)num.get(b.getAtom(0))).intValue();
					int a2 = ((Integer)num.get(b.getAtom(1))).intValue();
					printer.print(StringUtils.leftPadded((new Integer(counter)).toString(), 5, " ") + " ");
					printer.print(StringUtils.leftPadded((new Integer(a1)).toString(), 5, " ") + " ");
					printer.print(StringUtils.leftPadded((new Integer(a2)).toString(), 5, " ") + " ");
					
					if (b.order == 1.5){
						printer.println("ar");
					}
					else if (b.order == 2.0){
						printer.println("2");
					}
					else if (b.order == 3.0){
						printer.println("3");
					}
					else{
						printer.println("1");
					}
					counter++;
				}
			}
			else{				
				int a1 = ((Integer)num.get(b.getAtom(0))).intValue();
				int a2 = ((Integer)num.get(b.getAtom(1))).intValue();
				printer.print(StringUtils.leftPadded((new Integer(counter)).toString(), 5, " ") + " ");
				printer.print(StringUtils.leftPadded((new Integer(a1)).toString(), 5, " ") + " ");
				printer.print(StringUtils.leftPadded((new Integer(a2)).toString(), 5, " ") + " ");
				
				if (b.order == 1.5){
					printer.println("ar");
				}
				else if (b.order == 2.0){
					printer.println("2");
				}
				else if (b.order == 3.0){
					printer.println("3");
				}
				else{
					printer.println("1");
				}
				counter++;
			}
		}
		printer.println("@<TRIPOS>SUBSTRUCTURE");
//		System.out.println("factor = " + factor);
		if (structureMap.getResidueCount() > 1){
			
			if (rename){
				//there should be only one residue
				Residue r = structureMap.getResidue(0);
				printer.print(StringUtils.leftPadded((new Integer(r.getResidueId())).toString(), 7, " ") + " ");
				printer.print(StringUtils.rightPadded(r.getCompoundCode(), 10, " ") + " ");
				
				//get the number of the beginning atom
				printer.print(StringUtils.leftPadded((new Integer(((Integer)num.get(r.getAtom(0))).intValue() - factor)).toString(), 9, " ") + " RESIDUE ");

				//no idea what this field is due to the lack of example files
				printer.print(StringUtils.leftPadded("0", 12, " ") + " ");
				printer.print(StringUtils.rightPadded(r.getChainId(), 4, " ") + "     ");
				printer.print(StringUtils.rightPadded(r.getCompoundCode(), 7, " ") + "\n");
			}
			else{
			
				for (int i = 0; i < structureMap.getResidueCount(); i++){
					Residue r = structureMap.getResidue(i);
					if (selection){
						boolean save = false;
						//scan through the atoms and if at least one is selected, write it out
						for (int j = 0; j < r.getAtomCount(); j++){
							Atom a = r.getAtom(j);
							if (styles.isSelected(a)){
								save = true;
								break;
							}
						}
						
						if (!save) continue;
					}
					printer.print(StringUtils.leftPadded((new Integer(r.getResidueId())).toString(), 7, " ") + " ");
					printer.print(StringUtils.rightPadded(r.getCompoundCode(), 10, " ") + " ");
					
					//get the number of the beginning atom
					printer.print(StringUtils.leftPadded((new Integer(((Integer)num.get(r.getAtom(0))).intValue() - factor)).toString(), 9, " ") + " RESIDUE ");
	
					//no idea what this field is due to the lack of example files
					printer.print(StringUtils.leftPadded("0", 12, " ") + " ");
					printer.print(StringUtils.rightPadded(r.getChainId(), 4, " ") + "     ");
					printer.print(StringUtils.rightPadded(r.getCompoundCode(), 7, " ") + "\n");
					
				}
			}
			
			printer.println("@<TRIPOS>SET");
		}	
			
			
		printer.close();
		
		
		
	}

	/**
	 * Flag fragment denotes whether this string will be used to write out a custom fragment. In that case, some lines are
	 * omitted, and no substructure information is saved. StructureViewer, when non-null, is used to check whether atoms
	 * are part of an independently moved group. In that case, their transformed coordinates are saved, instead of the recorded ones
	 * @param structure
	 * @param fragment
	 * @return
	 */
	public static String convertMol2ToString(Structure structure, boolean fragment, StructureViewer viewer, HashMap atomMap) {
		
		//first, check if the data is with bond order info
		StructureMap structureMap = structure.getStructureMap();
//		if (structureMap.bondOrderImported || structureMap.bondOrderUserAssigned){
		
			//just in case, keep a reference map of initial atom numbers vs written ones 
			//this is necessary if atom numbers in the structure are not consecutive
			HashMap num = new HashMap();
			
			StringBuffer buffer = new StringBuffer();
			
			if (!fragment){
				buffer.append("#\n");
				buffer.append("#\tCreating user name:\tSirius export\n");
				buffer.append("#\tCreation time:\t\t" + Calendar.getInstance().getTime() + "\n\n");
				
				buffer.append("@<TRIPOS>MOLECULE\n");
				if (viewer != null){
					buffer.append(viewer.getStructureName(structure) + "\n");
				}
				else{
					buffer.append(structure.getUrlString());
					buffer.append("\n");
				}
				buffer.append(structureMap.getAtomCount() + " " + structureMap.getBondCount());
				if (structureMap.getResidueCount() > 1){
					buffer.append(" " + structureMap.getResidueCount() + "\n");
				}
				else{
					buffer.append("\n");
				}
				buffer.append("SMALL\n");
				
				if (structureMap.chargesUserAssigned){
					buffer.append("USER_CHARGES\n");
				}
				else{
					buffer.append("****\n");
				}
			}
			
			HashMap transformed = null;
			if (viewer != null && viewer.getTransformedCoordinates().size() > 0){
				transformed = viewer.getTransformedCoordinates();
			}
			
			//determine whether atom numbering needs to be rolled back to 1
			int start = structureMap.getAtom(0).number;
			int factor = start - 1;
			
			if (fragment){
				buffer.append("@ATOM\n");
			}
			else{
				buffer.append("@<TRIPOS>ATOM\n");
			}
			
			int count = 1;
			
			for (int i = 0; i < structureMap.getChainCount(); i++){
				Chain chain = structureMap.getChain(i);
				
				for (int j = 0; j < structureMap.getResidueCount(); j++){
					Residue r = structureMap.getResidue(j);
					if (!r.getChainId().equals(chain.getChainId()))continue;
					for (int k = 0; k < r.getAtomCount(); k++){
						Atom a = r.getAtom(k);

						num.put(a, new Integer(count));
						if (atomMap != null) atomMap.put(a, new Integer(count-1));
						buffer.append(StringUtils.leftPadded((new Integer(count)).toString(), 7, " ") + " ");
						
						buffer.append(StringUtils.rightPadded(a.name, 8, " ") + " ");
					
						String x = null;
						String y = null;
						String z = null;
						
						if (transformed != null && transformed.containsKey(a)){
							x = (new Double(((double[])transformed.get(a))[0])).toString();
							y = (new Double(((double[])transformed.get(a))[1])).toString();
							z = (new Double(((double[])transformed.get(a))[2])).toString();
						}
						else{
							x = (new Double(a.coordinate[0])).toString();
							y = (new Double(a.coordinate[1])).toString();
							z = (new Double(a.coordinate[2])).toString();
						}
						
						//make sure the part of the number after the point has four digits
						int index = x.indexOf(".");
						String subX = x.substring(0, index+1);
						String ext = x.substring(index+1);
						x = subX + StringUtils.rightPadded(ext, 4, "0");
						
						index = y.indexOf(".");
						String subY = y.substring(0, index+1);
						ext = y.substring(index+1);
						y = subY + StringUtils.rightPadded(ext, 4, "0");

						index = z.indexOf(".");
						String subZ = z.substring(0, index+1);
						ext = z.substring(index+1);
						z = subZ + StringUtils.rightPadded(ext, 4, "0");

						buffer.append(StringUtils.leftPadded(x, 9, " ") + " ");
						buffer.append(StringUtils.leftPadded(y, 9, " ") + " ");
						buffer.append(StringUtils.leftPadded(z, 9, " ") + " ");
					
					
						//form the element type string
						if (a.element == 1){
							buffer.append(StringUtils.rightPadded("H", 6, " ") + " ");
						}
						else{
							Vector b = structureMap.getBonds(a);
//							System.out.println("b = " + b + " for " + a.name);
							if (a.element == 7){
								//check for explicit hybridization record
								if (a.hybridization == Atom.SP2){
									buffer.append(StringUtils.rightPadded("N.2", 6, " ") + " ");
								}
								else{
									if (a.name.equals("N") && structureMap.getResidue(a).getClassification() == Residue.COMPOUND_AMINO_ACID){
										//check for N-terminus
										Vector aa = structureMap.getAtoms(a);
										if (aa != null){
											int counter = 0;
											for (int m = 0; m < aa.size(); m++){
												if (((Atom)aa.get(m)).element == 1){
													counter++;
												}
											}
											if (counter == 2){
												buffer.append(StringUtils.rightPadded("N.3", 6, " ") + " ");
											}
											else{
												buffer.append(StringUtils.rightPadded("N.am", 6, " ") + " ");
											}
										}
										else{
											buffer.append(StringUtils.rightPadded("N.3", 6, " ") + " ");
										}
									}
									else if (b != null && b.size() == 3){
										buffer.append(StringUtils.rightPadded("N.3", 6, " ") + " ");
									}
									else{
										buffer.append(StringUtils.rightPadded("N", 6, " ") + " ");
									}
								}
							}
							else{
								String type = a.getElement() + ".3";
								//if hybridization different from SP3 is known, just record it
								if (a.hybridization == Atom.SP2){
									type = a.getElement() + ".2";
								}
								else{
									//check for bond orders around the atom
									if (b != null){
										for (int m = 0; m < b.size(); m++){
											if (a.element == 6 && ((Bond)b.get(m)).order == 1.5){
												type = "C.ar";
												break;
											}
											if (((Bond)b.get(m)).order == 2){
												type = a.getElement() + ".2";
												break;
											}
										}
									}
								}
								buffer.append(StringUtils.rightPadded(type, 6, " ") + " ");
							}
						}
						
						buffer.append(StringUtils.leftPadded((new Integer(a.residue_id)).toString(), 4, " ") + " ");
						
						buffer.append(StringUtils.rightPadded(a.compound, 8, " ") + " ");
						
						String q = (new Double(a.partialCharge)).toString();
						index = q.indexOf(".");
						String subQ = q.substring(0, index+1);
						String ex = q.substring(index+1);
						q = subQ + StringUtils.rightPadded(ex, 4, "0");

						buffer.append(StringUtils.leftPadded(q, 8, " ") + " ");
						
						if (structureMap.getResidueCount() > 1){
							if (a.backbone){
								buffer.append("BACKBONE\n");
							}
							else{
								buffer.append("****\n");
							}
						}
						else{
							buffer.append("\n");
						}
						
						count++;
					}
				}
			}
			
			if (fragment){
				buffer.append("@BOND\n");
			}
			else{
				buffer.append("@<TRIPOS>BOND\n");
			}
			
			int counter = 1;
			for (int i = 0; i < structureMap.getBondCount(); i++){
				Bond b = structureMap.getBond(i);
				int a1 = ((Integer)num.get(b.getAtom(0))).intValue();
				int a2 = ((Integer)num.get(b.getAtom(1))).intValue();
				buffer.append(StringUtils.leftPadded((new Integer(counter)).toString(), 5, " ") + " ");
				buffer.append(StringUtils.leftPadded((new Integer(a1)).toString(), 5, " ") + " ");
				buffer.append(StringUtils.leftPadded((new Integer(a2)).toString(), 5, " ") + " ");
				
				if (b.order == 1.5){
					buffer.append("ar\n");
				}
				else if (b.order == 2.0){
					buffer.append("2\n");
				}
				else if (b.order == 3.0){
					buffer.append("3\n");
				}
				else{
					buffer.append("1\n");
				}
				
				counter++;
			}
			
			if (!fragment){
				buffer.append("@<TRIPOS>SUBSTRUCTURE\n");
				
				if (structureMap.getResidueCount() > 1){
					for (int i = 0; i < structureMap.getResidueCount(); i++){
						Residue r = structureMap.getResidue(i);
	//					System.out.println("residue = " + r + ", 0 atom = " + r.getAtom(0).name + " with n = " + r.getAtom(0).number);
						buffer.append(StringUtils.leftPadded((new Integer(r.getResidueId())).toString(), 7, " ") + " ");
						buffer.append(StringUtils.rightPadded(r.getCompoundCode(), 10, " ") + " ");
						//get the number of the beginning atom
						buffer.append(StringUtils.leftPadded((new Integer(((Integer)num.get(r.getAtom(0))).intValue() - factor)).toString(), 9, " ") + " RESIDUE ");
						
						//no idea what this field is due to the lack of example files
						buffer.append(StringUtils.leftPadded("0", 12, " ") + " ");
						buffer.append(StringUtils.rightPadded(r.getChainId(), 1, " ") + "     ");
						buffer.append(StringUtils.rightPadded(r.getCompoundCode(), 10, " ") + "\n");
						
					}
					
					buffer.append("@<TRIPOS>SET\n");
				}	
			}
			
				
//		}
//		else{
//			parent.displayErrorMessage("Structure has no bond order information. MOL2 format cannot be written");
//			return;
//		}
		
		return buffer.toString();
		
	}
	
	public static Structure readPDB(File file){
		
		PdbEntryLoader loader = new PdbEntryLoader();
		if (!loader.canLoad(file)) return null;
		
		StructureEntry e = (StructureEntry)loader.load(file, -1);
		return e.getStructure();
		
	}
	
	public static Entry loadSDFFromString(String content, int index){
		
		StructureEntry entry = new StructureEntry();
		Structure structure = loadSDFStructure( content, index );
		
//		System.out.println("Structure = " + structure);

		if ( structure == null ){
			return null;
		}
		
		entry.setStructure(structure);
		entry.setName(structure.getUrlString());
		entry.setSourceUrl(null);
		
		return entry;
	}
	
	public static Structure loadSDFStructure( String content, int index ){
		
		Structure structure = null;
		
//		System.out.println(content);
		
		if ( content == null ) return null;
		
		passComponents = new Hashtable();
		
		int linePos = 0;
		int bytesRead = 0;
		int lastRead = 0;
		int lines = 0;
		int readCount = 0;

		boolean atomFlag = false;
		boolean bondFlag = false;
		boolean moleculeFlag = false;
		
		int lineNumber = 0;
		boolean newMolecule = true;
		name = null;
		
		int counter = 0;
		
		int atomCount = 0;
		int bondCount = 0;
		
		int atomCounter = 0;
		int bondCounter = 0;
		
		int startLine = -1;
		
		boolean terminate = false;//force end of line
		
		try
		{
			String[] data = content.split("\\n");
			for (int i = 0; i < data.length; i++){
				String token = data[i];
				
//				System.out.println("token = #" + token + "#");
				
				lines++;
				linePos = 0;
				lineNumber++;
				terminate = false;

				if (token.equals("\n")){
					continue;
				}
				
				
				
				if (newMolecule && name == null){
					if (counter == index){
						//this is the first line of a molecule record: get the name
						name = token;
						name = name.trim();
						if (name == null || name.length() == 0){
							name = "SDF" + counter;
						}
						startLine = lineNumber + 3;
						
//						System.out.println("name = " + name);
						
//						System.out.println("startLine = " + startLine);
					}
					
				}
				
				if (newMolecule){
//					System.out.println("lineNumber = " + lineNumber + " at " + token);
					if (counter == index && lineNumber == startLine){
						//check if this is the stats line
						String temp = token;
						temp = temp.trim();
//						System.out.println("Temp = " + temp);
						try{
							atomCount =  Integer.parseInt(token.substring(0, 3).trim() );
							bondCount =  Integer.parseInt(token.substring(3,6).trim() );
//							System.out.println("ac = " + atomCount + ", bc = " + bondCount);
							atomFlag = true;
							newMolecule = false;
						}
						catch (NumberFormatException ex){
							System.out.println("erreur =" + ex);
							ex.printStackTrace();
							//not yet
						}
					}
				}
				
				
						
						
				if (token.equals("$$$$")){ 
					
					if (counter == index){

						//the molecule flag has ended
						//create structure for it
						structure = new Structure()
						{
							// A hashtable of vectors where
							// each hash KEY is the StructureComponent type String.
							// each hash VALUE is a Vector of StructureComponent objects.
							protected Hashtable structureComponents = null;

							// To free up the global state for another load call.
							private String localUrlString;

							// public Structure()  Anonymous inner class constructor.
							{
								structureComponents = passComponents;
								passComponents = new Hashtable();

								localUrlString = name;
								
							}

							public String getUrlString( ){
								return localUrlString;
							}

							
							public int getStructureComponentCount( String scType ){
								Vector records = (Vector) structureComponents.get( scType );
								if ( records == null )
									return 0;
								else
									return records.size( );
							}

							private void getStructureComponentByIndex( int index,
								StructureComponent structureComponent )
								throws IndexOutOfBoundsException, IllegalArgumentException
							{
								String type = structureComponent.getStructureComponentType();
								if ( type == null )
									throw new IllegalArgumentException( "type inconnu" + type );

								Vector records = (Vector) structureComponents.get( type );
								if ( records == null )
									throw new IllegalArgumentException( "pas d\'enregistrement du type " + type );

								StructureComponent cachedComponent =
									(StructureComponent) records.elementAt( index );

								cachedComponent.structure = this;
								structureComponent.structure = this;
								structureComponent.copy( cachedComponent );
				            }

							public StructureComponent getStructureComponentByIndex( String type,
								int index )
								throws IndexOutOfBoundsException, IllegalArgumentException
							{
								Vector records = (Vector) structureComponents.get( type );
								if ( records == null )
									throw new IllegalArgumentException( "pas d\'enregistrement du type " + type );

								StructureComponent structureComponent =
									(StructureComponent) records.elementAt( index );
								structureComponent.structure = this;

								return structureComponent;
				            }
						};

						atomFlag = false;
						bondFlag = false;
						
						atomCounter = 0;
						bondCounter = 0;
						
						return structure;
					}
							
					counter++;
					
					break;
				}
						
				if (atomFlag && atomCounter == atomCount){
//					System.out.println("resetting to bonds");
					atomFlag = false;
					bondFlag = true;
				}
				
				if (bondFlag && bondCounter == bondCount){
//					System.out.println("bond counter = " + bondCounter);
					bondFlag = false;
				}

						
				if (atomFlag){
					//read the current line and create an atom record
//					System.out.println("Atom found");
					try{
						Atom atom = new Atom( );

						atom.number =  atomCounter;

						atom.setElement(token.substring(31,34).trim());
						if (!PeriodicTable.isElement(atom.element)){
							atom.element = 6;//default to C
						}
						
//						System.out.println("element = " + atom.element);
						
						atom.name = atom.getElement() + atomCounter;
						
						atom.chain_id = "_";
						atom.compound = "XXX";
						
						int charge = Integer.parseInt(token.substring(36, 39).trim());
						if (charge == 0) atom.partialCharge = 0.0f;
						if (charge == 1) atom.partialCharge = 3.0f;
						if (charge == 2) atom.partialCharge = 2.0f;
						if (charge == 3) atom.partialCharge = 1.0f;
						if (charge == 4) atom.partialCharge = 0.0f;
						if (charge == 5) atom.partialCharge = -1.0f;
						if (charge == 6) atom.partialCharge = -2.0f;
						if (charge == 7) atom.partialCharge = -3.0f;
						
						
						atom.coordinate = new double[3];
						atom.coordinate[0] =
							Double.parseDouble( token.substring(0, 10).trim() );
						atom.coordinate[1] =
							Double.parseDouble( token.substring(10,20).trim() );
						atom.coordinate[2] =
							Double.parseDouble( token.substring(20,30).trim() );
						
//								System.out.println("atom: " + atom.name + " " + atom.element + ": charge = " + atom.partialCharge + 
//										", x = " + atom.coordinate[0] + ", y = " + atom.coordinate[1] + ", z = " + atom.coordinate[2]);

						Vector records = (Vector) passComponents.get(
							StructureComponentRegistry.TYPE_ATOM );
						if ( records == null )
						{
							records = new Vector( );
							passComponents.put(
								StructureComponentRegistry.TYPE_ATOM, records );
						}
						records.add( atom );
						atomCounter++;
					}
					catch (Exception ex){
						continue;
					}
					
					
				}
						
				if (bondFlag){
					
					
					int n1 = Integer.parseInt(token.substring(0, 3).trim()) - 1;
					int n2 = Integer.parseInt(token.substring(3, 6).trim()) - 1;
					
//					System.err.println(n1 + " - " + n2);
					
					String order = token.substring(6, 9).trim();
//					System.out.println("order = #" + order + "#");
					
					//get the right atoms from the records
					Atom a1 = (Atom)((Vector) passComponents.get(StructureComponentRegistry.TYPE_ATOM )).get(n1);
					Atom a2 = (Atom)((Vector) passComponents.get(StructureComponentRegistry.TYPE_ATOM )).get(n2);
					
					Bond bond = new Bond(a1, a2);
					if (order.startsWith("1")){
						bond.setOrderOnly(1.0f);//the default is already SP3 and atoms with another value should not be touched
					}
					else if (order.startsWith("2")){
						bond.setOrderOnly(2.0f);
						a1.hybridization = Atom.SP2;
						a2.hybridization = Atom.SP2;
					}
					else if (order.startsWith("3")){
						bond.setOrderOnly(3.0f);
						a1.hybridization = Atom.SP;
						a2.hybridization = Atom.SP;
					}
					else if (order.startsWith("4")){
						bond.setOrderOnly(1.5f);
						
						a1.hybridization = Atom.SP2;
						a2.hybridization = Atom.SP2;
					}
					
					Vector records = (Vector) passComponents.get(
							StructureComponentRegistry.TYPE_BOND );
						if ( records == null )
						{
							records = new Vector( );
							passComponents.put(
								StructureComponentRegistry.TYPE_BOND, records );
						}
					records.add( bond );
					bondCounter++;
					
				}
				
			}
		}
		catch( Exception e )
		{
			e.printStackTrace();
			System.out.println("Erreur pour " + name);
			return null;
		}

		name = null;
		
		System.out.println("structure retourne =" + structure);
		
		return structure;
	}
	
	public static Entry loadMol2FromString(String content){
		
		StructureEntry entry = new StructureEntry();
		Structure structure = loadMol2Structure( content );

		if ( structure == null ){
			return null;
		}
		
		entry.setStructure(structure);
		entry.setName(structure.getUrlString());
		entry.setSourceUrl(null);
		
		return entry;
	}
	
	/**
	 * This method reads coordinate data. But in addition to standard mol2, the records may also contain
	 * information on conformation.
	 * @param content
	 * @return
	 */
	private static Structure loadMol2Structure( String content ){

		if ( content == null ){
			return null;
		}
		
		passComponents = new Hashtable( );
		
//		System.out.println("Loading mol2 from string");


		int linePos = 0;
		int bytesRead = 0;
		int lastRead = 0;
		int lines = 0;
		int readCount = 0;

		boolean atomFlag = false;
		boolean bondFlag = false;
		boolean moleculeFlag = false;
		boolean substructureFlag = false;
		boolean conformationFlag = false;
		
		int tempAtomNum = 0;//used to count atom offsets of residue records
		int tempResNum = 0;
		String tempChainId = null;
		String tempRes = null;
		
		int nameLine = 0;
		int lineNumber = -1;//once it hits the first line, it becomes 0
		
		//variables to handle different formats of bond records
		boolean first = true;
		int startAtom1 = -1;
		int spanAtom1 = -1;
		int startAtom2 = -1;
		int spanAtom2 = -1;
		int startOrder = -1;
		int spanOrder = -1;

		try
		{
			StringTokenizer tok = new StringTokenizer(content, "\n", false);
			while (tok.hasMoreTokens()){
				lineNumber++;
				String token = tok.nextToken();
//				System.out.println("Token = " + token);
				char[] line = token.toCharArray();
				
						int atomCount = 0;
						int bondCount = 0;
						
						if (moleculeFlag){
							if (lineNumber == nameLine){
								name = token.trim();
							}
						}
						
						//check the flags in the line
						if (
							(line[0] == '@') &&
							(line[9] == 'M') &&
							(line[10] == 'O') &&
							(line[11] == 'L') &&
							(line[12] == 'E') &&
							(line[13] == 'C') &&
							(line[14] == 'U') &&
							(line[15] == 'L') &&
							(line[16] == 'E')
						){
//							System.out.println("molecule flag");
							//the molecule flag has started
							//get the atom and bond numbers (later)
//							byte[] line_temp = (byte[])data.get(i);
//							name = new String(line_temp);
//							System.out.println("name = " + name);
							moleculeFlag = true;
							nameLine = lineNumber + 1;
							continue;
						}
							
						if (
							(line[0] == '@') &&
							(line[9] == 'A') &&
							(line[10] == 'T') &&
							(line[11] == 'O') &&
							(line[12] == 'M')
						){
//							System.out.println("Atom flag found");
							//the atom flag has started
							atomFlag = true;
							moleculeFlag = false;
							continue;
						}
						
						if (
								(line[0] == '@') &&
								(line[9] == 'B') &&
								(line[10] == 'O') &&
								(line[11] == 'N') &&
								(line[12] == 'D')
						){
							//the molecule flag has started
							bondFlag = true;
							atomFlag = false;
							continue;
						}
						
						
						if (
								(line[0] == '@') &&
								(line[9] == 'S') &&
								(line[10] == 'U') &&
								(line[11] == 'B') &&
								(line[12] == 'S') &&
								(line[13] == 'T') &&
								(line[14] == 'R') &&
								(line[15] == 'U') &&
								(line[16] == 'C')
						){
							//the molecule flag has started
							bondFlag = false;
							substructureFlag = true;
							continue;
						}
						
						if (
								(line[0] == '@') &&
								(line[9] == 'C') &&
								(line[10] == 'O') &&
								(line[11] == 'N') &&
								(line[12] == 'F') &&
								(line[13] == 'O') &&
								(line[14] == 'R') &&
								(line[15] == 'M') 
						){
							//the molecule flag has started
							substructureFlag = false;
							conformationFlag = true;
							continue;
						}

						if (atomFlag){
							//read the current line and create an atom record
//							System.out.println("Atom found");
							try{
								Atom atom = new Atom( );
	
								atom.number =  Integer.parseInt( (new String( line, 0, 7 )).trim() );
	
//								System.out.print(atom.number + "\t");
								String block = (new String( line, 47, 6 )).trim();
								int dot = block.indexOf(".");
								if (dot == -1){
									atom.setElement(block);
								}
								else{
									atom.setElement(block.substring(0, dot));
								}
								
								
//								System.out.print(atom.element + "\t");
								
								atom.name = (new String( line, 8, 4 )).trim();
//								System.out.print(atom.name + "\t");
								
//								
								try{
									String temp = (new String( line, 59, 8 )).trim();
									temp.replace('.', '_');
									if (temp != null && temp.length() >= name.length() && temp.startsWith(name)){
										atom.compound = name;
									}
									else{
										//it's probably a residue name
										try{
											atom.compound = temp.substring(0,3);
										}
										catch (Exception e){
											atom.compound = name;
										}
									}
								}
								catch (Exception exx){
//									exx.printStackTrace();
									atom.compound = "XXX";
								}
//								System.out.print(atom.compound + "\t");
								
								atom.chain_id = "_";
								
								atom.residue_id = Integer.parseInt((new String( line, 53, 5 )).trim());
//								System.out.print(atom.residue_id + "\t");
								
								atom.partialCharge = Float.parseFloat((new String( line, 69, 7 )).trim());
								
								atom.coordinate = new double[3];
								atom.coordinate[0] =
									Double.parseDouble( (new String( line, 16, 10 )).trim() );
	//							System.out.print(atom.coordinate[0] + "\t");
								atom.coordinate[1] =
									Double.parseDouble( (new String( line, 26, 10 )).trim() );
	//							System.out.print(atom.coordinate[1] + "\t");
								atom.coordinate[2] =
									Double.parseDouble( (new String( line, 36, 10 )).trim() );
	//							System.out.print(atom.coordinate[2] + "\t\t");
	
//								System.out.print(atom.partialCharge + "\n");
								
//								atom.bfactor = 0.0f;
//								atom.occupancy = 1.0f;
								
								Vector records = (Vector) passComponents.get(
									StructureComponentRegistry.TYPE_ATOM );
								if ( records == null )
								{
									records = new Vector( );
									passComponents.put(
										StructureComponentRegistry.TYPE_ATOM, records );
								}
								records.add( atom );
//								System.out.println("Added " + atom.name + " = " + atom.element + " to size " + records.size());
							}
							catch (Exception ex){
								ex.printStackTrace();
								continue;
							}
							
							
						}
						
						if (bondFlag){
							
/*							if (first){
								
								//check for the format of the bond records
								String sample = token;
								char[] set = sample.toCharArray();
								for (int i = 0; i < set.length; i++){
									if (i == 0) continue;
									if (set[i] == ' ' && set[i-1] != ' '){
										if (startAtom1 == -1){
											startAtom1 = i;
										}
										else if (startAtom2 == -1){
											startAtom2 = i;
											spanAtom1 = i - startAtom1;
										}
										else if (startOrder == -1){
											startOrder = i+1;
											spanAtom2 = startOrder - startAtom2;
										}
									}
								}
								spanOrder = 2;
								
								first = false;
							}
*/							
//							System.out.println("token = #" + token + "#");
							String temp = token.trim().substring(token.trim().indexOf(" ")).trim();//everything except the number of the bond
//							System.out.println("temp = #" + temp + "#");
							
							String nn1 = temp.substring(0, temp.indexOf(" ")).trim();
							String temp2 = temp.substring(temp.indexOf(" ")).trim();
							String nn2 = temp2.substring(0, temp2.indexOf(" ")).trim();
							String order = temp2.substring(temp2.indexOf(" ")).trim();
							
//							System.out.println("temp2 = " + temp2);
//							System.out.println("nn1 = " + nn1);
//							System.out.println("nn2 = " + nn2);
//							System.out.println("order = " + order);
							
							int n1 = Integer.parseInt(nn1) - 1;
							int n2 = Integer.parseInt(nn2) - 1;
							
//							int n1 = (Integer.parseInt(token.substring(startAtom1, startAtom1 + spanAtom1).trim())) - 1;
//							int n2 = (Integer.parseInt(token.substring(startAtom2, startAtom2 + spanAtom2).trim())) - 1;
							
//							String order = token.substring(startOrder);
							
							//get the right atoms from the records
							Atom a1 = (Atom)((Vector) passComponents.get(
									StructureComponentRegistry.TYPE_ATOM )).get(n1);
							Atom a2 = (Atom)((Vector) passComponents.get(
									StructureComponentRegistry.TYPE_ATOM )).get(n2);
							
							Bond bond = new Bond(a1, a2);
							if (order.startsWith("1")){
								bond.setOrderOnly(1.0f);//the default is already SP3 and atoms with another value should not be touched
							}
							else if (order.startsWith("2")){
								bond.setOrderOnly(2.0f);
								a1.hybridization = Atom.SP2;
								a2.hybridization = Atom.SP2;
							}
							else if (order.startsWith("3")){
								bond.setOrderOnly(3.0f);
								a1.hybridization = Atom.SP;
								a2.hybridization = Atom.SP;
							}
							else if (order.startsWith("ar")){
								bond.setOrderOnly(1.5f);
								
								a1.hybridization = Atom.SP2;
								a2.hybridization = Atom.SP2;
							}
							
							Vector records = (Vector) passComponents.get(
									StructureComponentRegistry.TYPE_BOND );
								if ( records == null )
								{
									records = new Vector( );
									passComponents.put(
										StructureComponentRegistry.TYPE_BOND, records );
								}
							records.add( bond );
							
						}
						
						if (substructureFlag){
							
							if (line[0] == '@'){
								
								//fill out the last residue
								Vector atoms = (Vector) passComponents.get(
										StructureComponentRegistry.TYPE_ATOM );
								for (int k = tempAtomNum-1; k < atoms.size(); k++){
									try{
										Atom a = (Atom)atoms.get(k);
										a.chain_id = tempChainId;
										a.residue_id = tempResNum;
										a.compound = tempRes;
									}
									catch (Exception e){
//										System.out.println("Exception = " + e);
										continue;
									}
								}
								substructureFlag = false;
								continue;
							}
							
							//read the records from substructure section
							int num = Integer.parseInt(new String(line, 0, 7).trim());
							int anum = Integer.parseInt(new String(line, 22, 6).trim());
							String chain = new String( line, 50, 2).trim();
							String res = new String( line, 56, 3).trim();
							
							//this is the first record
							if (tempRes == null){
								tempRes = res;
								tempChainId = chain;
								tempAtomNum = anum;
								tempResNum = num;
								continue;
							}
							
							
							Vector atoms = (Vector) passComponents.get(
									StructureComponentRegistry.TYPE_ATOM );
							for (int k = tempAtomNum-1; k < anum; k++){
								try{
									Atom a = (Atom)atoms.get(k);
									a.chain_id = tempChainId;
									a.residue_id = tempResNum;
									a.compound = tempRes;
								}
								catch (Exception e){
//									System.out.println("Exception = " + e);
									continue;
								}
//								System.out.println(a.number + ":" + a.compound + "," + a.residue_id + "," + a.chain_id);
							}
							
							tempRes = res;
							tempChainId = chain;
							tempAtomNum = anum;
							tempResNum = num;
							
						}
						
						if (conformationFlag){
							
							if (
									(line[0] == 'H') &&
									(line[1] == 'E') &&
									(line[2] == 'L') &&
									(line[3] == 'I') &&
									(line[4] == 'X')
								)
								{
								
									Helix helix = new Helix( );
									
									String[] data = new String[7];
									String record = (new String(line)).trim();
									StringTokenizer tk = new StringTokenizer(record, "\t", false);
									int counter = 0;
									while (tk.hasMoreTokens()){
										data[counter] = tk.nextToken();
										counter++;
									}
			
									helix.name = data[0];
									helix.start_compound = data[1];
									helix.start_chain = data[2];
									if ( helix.start_chain == null ) helix.start_chain = "_";
									helix.start_residue = Integer.parseInt(data[3]);
									helix.end_compound = data[4];
									helix.end_chain = data[5];
									if ( helix.end_chain == null ) helix.end_chain = "_";
									helix.end_residue = Integer.parseInt( data[6] );
									helix.righthand = true;
			
									Vector records = (Vector) passComponents.get(
										StructureComponentRegistry.TYPE_HELIX );
									if ( records == null )
									{
										records = new Vector( );
										passComponents.put(
											StructureComponentRegistry.TYPE_HELIX, records );
									}
									records.add( helix );
			
									// Reset linePos to the start of the line buffer.
									linePos = 0;
									continue;
								}

							if (
									(line[0] == 'S') &&
									(line[1] == 'H') &&
									(line[2] == 'E') &&
									(line[3] == 'E') &&
									(line[4] == 'T')
								)
								{
									Strand strand = new Strand( );
									
									String[] data = new String[7];
									String record = new String(line);
									if (record.indexOf("\n") != -1) record = record.substring(0, record.indexOf("\n"));
									StringTokenizer tk = new StringTokenizer(record, "\t", false);
									int counter = 0;
									while (tk.hasMoreTokens()){
										data[counter] = tk.nextToken();
										counter++;
									}
			
									strand.name = data[0];
									strand.start_compound = data[1];
									strand.start_chain = data[2];
									if ( strand.start_chain == null ) strand.start_chain = "_";
									strand.start_residue = Integer.parseInt(data[3]);
									strand.end_compound = data[4];
									strand.end_chain = data[5];
									if ( strand.end_chain == null ) strand.end_chain = "_";
									strand.end_residue = Integer.parseInt( data[6] );
			
									Vector records = (Vector) passComponents.get(
										StructureComponentRegistry.TYPE_STRAND );
									if ( records == null )
									{
										records = new Vector( );
										passComponents.put(
											StructureComponentRegistry.TYPE_STRAND, records );
									}
									records.add( strand );
			
									// Reset linePos to the start of the line buffer.
									linePos = 0;
									continue;
								}
							
							if (
									(line[0] == 'T') &&
									(line[1] == 'U') &&
									(line[2] == 'R') &&
									(line[3] == 'N') 
								)
								{
									Turn turn = new Turn( );
									
									String[] data = new String[7];
									String record = new String(line);
									int end = record.indexOf("\n");
									if (end > 0){
										record = record.substring(0, record.indexOf("\n"));
									}
									else{
										record = record.substring(0);
									}
									StringTokenizer tk = new StringTokenizer(record, "\t", false);
									int counter = 0;
									while (tk.hasMoreTokens()){
										data[counter] = tk.nextToken();
										counter++;
									}
			
									turn.name = data[0];
									turn.start_compound = data[1];
									turn.start_chain = data[2];
									if ( turn.start_chain == null ) turn.start_chain = "_";
									turn.start_residue = Integer.parseInt(data[3]);
									turn.end_compound = data[4];
									turn.end_chain = data[5];
									if ( turn.end_chain == null ) turn.end_chain = "_";
									turn.end_residue = Integer.parseInt( data[6] );
			
									Vector records = (Vector) passComponents.get(
										StructureComponentRegistry.TYPE_TURN );
									if ( records == null )
									{
										records = new Vector( );
										passComponents.put(
											StructureComponentRegistry.TYPE_TURN, records );
									}
									records.add( turn );
			
									// Reset linePos to the start of the line buffer.
									linePos = 0;
									continue;
								}
							
						}

			}
		}
		catch( Exception e )
		{
			e.printStackTrace();
//			Status.progress( 1.0f, null );
			return null;
		}
			
		
		//
		// Create the Structure object
		//
		Structure structure = new Structure()
		{
			// A hashtable of vectors where
			// each hash KEY is the StructureComponent type String.
			// each hash VALUE is a Vector of StructureComponent objects.
			protected Hashtable structureComponents = null;

			// To free up the global state for another load call.
			private String localUrlString;

			// public Structure()  Anonymous inner class constructor.
			{
				structureComponents = passComponents;
				passComponents = null;

				localUrlString = name;
				
			}

			public String getUrlString( ){
				return localUrlString;
			}

			
			public int getStructureComponentCount( String scType ){
				Vector records = (Vector) structureComponents.get( scType );
				if ( records == null )
					return 0;
				else
					return records.size( );
			}

			private void getStructureComponentByIndex( int index,
				StructureComponent structureComponent )
				throws IndexOutOfBoundsException, IllegalArgumentException
			{
				String type = structureComponent.getStructureComponentType();
				if ( type == null )
					throw new IllegalArgumentException( "type inconnu" + type );

				Vector records = (Vector) structureComponents.get( type );
				if ( records == null )
					throw new IllegalArgumentException( "pas d\'enregistrement du type " + type );

				StructureComponent cachedComponent =
					(StructureComponent) records.elementAt( index );

				cachedComponent.structure = this;
				structureComponent.structure = this;
				structureComponent.copy( cachedComponent );
            }

			public StructureComponent getStructureComponentByIndex( String type,
				int index )
				throws IndexOutOfBoundsException, IllegalArgumentException
			{
				Vector records = (Vector) structureComponents.get( type );
				if ( records == null )
					throw new IllegalArgumentException( "pas d\'enregistrement du type " + type );

				StructureComponent structureComponent =
					(StructureComponent) records.elementAt( index );
				structureComponent.structure = this;

				return structureComponent;
            }
		};

		// Progress is done.
//		Status.progress( 1.0f, null );
		
		return structure;
	}
	
	public static Structure getStructure(Hashtable components, String n){
		
		passComponents = components;
		name = n;
		
		Structure structure = new Structure()
		{
			// A hashtable of vectors where
			// each hash KEY is the StructureComponent type String.
			// each hash VALUE is a Vector of StructureComponent objects.
			protected Hashtable structureComponents = null;

			// To free up the global state for another load call.
			private String localUrlString;

			// public Structure()  Anonymous inner class constructor.
			{
				structureComponents = passComponents;
				passComponents = null;

				localUrlString = name;
				
			}

			public String getUrlString( ){
				return localUrlString;
			}

			
			public int getStructureComponentCount( String scType ){
				Vector records = (Vector) structureComponents.get( scType );
				if ( records == null )
					return 0;
				else
					return records.size( );
			}

			private void getStructureComponentByIndex( int index,
				StructureComponent structureComponent )
				throws IndexOutOfBoundsException, IllegalArgumentException
			{
				String type = structureComponent.getStructureComponentType();
				if ( type == null )
					throw new IllegalArgumentException( "type inconnu" + type );

				Vector records = (Vector) structureComponents.get( type );
				if ( records == null )
					throw new IllegalArgumentException( "pas d\'enregistrement du type " + type );

				StructureComponent cachedComponent =
					(StructureComponent) records.elementAt( index );

				cachedComponent.structure = this;
				structureComponent.structure = this;
				structureComponent.copy( cachedComponent );
            }

			public StructureComponent getStructureComponentByIndex( String type,
				int index )
				throws IndexOutOfBoundsException, IllegalArgumentException
			{
				Vector records = (Vector) structureComponents.get( type );
				if ( records == null )
					throw new IllegalArgumentException( "pas d\'enregistrement du type " + type );

				StructureComponent structureComponent =
					(StructureComponent) records.elementAt( index );
				structureComponent.structure = this;

				return structureComponent;
            }
		};
		
		return structure;
	}

	public static Entry loadPDBFromString(String input, String name){
		
		StructureEntry entry = new StructureEntry();
		Structure structure = loadPDBStructure(input, name);

		if ( structure == null ){
			return null;
		}
		
		entry.setStructure(structure);
		entry.setName(structure.getUrlString());
		entry.setSourceUrl(null);
		
		return entry;

	}
		
	public static Structure loadPDBStructure( String content, String nnn ){
			
			if ( content == null ) return null;
			

			passComponents = new Hashtable( );

			int[][] conect = null;
			int hetatmCount = 0;
			
			Vector hetatms = new Vector();//the heteroatom numbers are stored in order they are in the file
			
			HashMap numberOrder = new HashMap();//the other way around: atom number -> position
			
			int bytesRead = 0;
			int lastRead = 0;
			int lines = 0;
			int readCount = 0;
			
			HashMap extraBonds = new HashMap();//root Atom -> array of atoms that are in bonds and non-regular bonds in the protein
			//stores Atom references
			
			boolean protein = false;
			boolean dna = false;
			
			//aminoacid parsing variables
			Atom lastC = null;//C-terminal carbon of the last residue (to connect to the next N)
			Atom currentC = null;
			
			Atom lastO = null;//for DNA
			Atom currentO = null;
			
			int lastResidueId = -1;//to track change of residues
			String lastResidueName = null;
			String lastChainId = null;
			String currentChainId = null;
			boolean chainChange = false;
			HashMap residueAtoms = new HashMap();//stores name->Atom pairs for the current residue
			int dif = 1;
			
			Vector ter = new Vector();//positions of TER records to be used in case there is no explicit chain labels
			
			//frequently, there is no explicit chain id, but there are TER records that delimit separate chains.
			//here, if there are no chain ids in the file, automatically assign id A, and increment to B (and so on) as TER records
			//are seen.
			//if no TER records are in the file, revert all atoms to the default "_"
			int chainIdCounter = 0;
			
			int modelCounter = 0;
			boolean load = true;
			
			int number = 1;
			
			final String urlString = nnn;
			
			boolean waterStarted = false;
			
			boolean stop = false;
			
			int newline = -1;//index of newline in the current line
			
			try	{
				
				String[] data = content.split("\\n");
				for (int i = 0; i < data.length; i++){
					String line = data[i];
					
					lines++;

					if (
							(line.startsWith("END"))
						){
							//finish
							stop = true;
							break;
						}


					//
					// ATOM and HETATM records
					//
					boolean isAtom = false;
					boolean isHetatm = false;
					if (
						(line.startsWith("ATOM"))
					)
						isAtom = true;
					else if (
						(line.startsWith("HETATM"))
					){
						isHetatm = true;
					}
					
					if (
							(line.startsWith("TER")) 
						){
						
						Vector records = (Vector) passComponents.get(StructureComponentRegistry.TYPE_ATOM );
						if ( records != null ){
							ter.add(records.get(records.size()-1));//last Atom before the TER record
						}
					}
					
					if ( isAtom || isHetatm)
					{
						// PDB File Atom Record offsets as documented by
						// http://www.rcsb.org/pdb/docs/format/pdbguide2.2/part_62.html
						// 1 -  6   RecordName
						// 7 - 11   serial
						// 12       -
						// 13 - 16  name
						// 17       altLoc
						// 18 - 20  resName
						// 21       -
						// 22       chainID
						// 23 - 26  resSeq
						// 27       iCode
						// 28 - 30  -
						// 31 - 38  x
						// 39 - 46  y
						// 47 - 54  z
						// 55 - 60  occupancy
						// 61 - 66  tempFactor
						// 67 - 72  -
						// 73 - 76  segID
						// 77 - 78  element
						// 79 - 80  charge
						// NOTE: In this application, we need to subtract 1 from
						// each index in order to match the 0-based array offsets.
						
						
						Atom atom = new Atom( );
						String str = null;

						atom.number =  Integer.parseInt( line.substring(6,11).trim() );
						if (isHetatm){
//									orderNumber.put(new Integer(hetatmCount), new Integer(atom.number));
							numberOrder.put(new Integer(atom.number), new Integer(hetatmCount));
//									System.out.println("index = " + hetatmCount + ", number = " + atom.number);
							hetatms.add(atom);
							hetatmCount++;
						}
						
						atom.name = line.substring(12,16).trim();
						
						if (line.length() > 78){
							String ene = line.substring(77,79).trim();
							atom.setElement(ene);
							if (!PeriodicTable.isElement(atom.element)){
								//look elsewhere for element name
								String en2 = line.substring(12, 14).trim();
								if (PeriodicTable.isElementSymbol(en2)){
									atom.setElement(en2);
								}
								else{
									//check if there is a leading number
									atom.setElement(en2.substring(1));
									if (!PeriodicTable.isElementSymbol(en2.substring(1))){
										atom.element = 6;
									}
								}
							}
						}
						else{
							String en2 = line.substring(12,14).trim();
							if (PeriodicTable.isElementSymbol(en2)){
								atom.setElement(en2);
							}
							else{
								//check if there is a leading number
								atom.setElement(en2.substring(1));
								if (!PeriodicTable.isElementSymbol(en2.substring(1))){
									atom.element = 6;
								}
							}
						}
						
//								System.out.println("name = " + atom.name);
						atom.compound = line.substring(17,20).trim();
						if (atom.compound == null) atom.compound = "UKN";
						
						//check for starting waters
						if (!waterStarted){
							if(atom.compound.equals("HOH") || atom.compound.equals("WAT")){
								waterStarted = true;
								Vector records = (Vector) passComponents.get(StructureComponentRegistry.TYPE_ATOM );
								if ( records != null ){
									//if ter record has not been found in the file, add one
									Object last = records.get(records.size()-1);
									if (!ter.contains(last)){
										ter.add(last);
									}
								}

							}
						}
						
						atom.chain_id = line.substring(21,22).trim();
						if ( atom.chain_id == null ){
							atom.chain_id = "_";
							
							//try to get the residue number from differently placed fields
							try{
								atom.residue_id =	Integer.parseInt(line.substring(22,26).trim() );
							}
							catch (Exception ex){
								atom.residue_id =	Integer.parseInt(line.substring(23,27).trim() );
							}
						}
						else if ( atom.chain_id.equals( "" ) ){
							atom.chain_id = "_";
							
							//try to get the residue number from differently placed fields
							try{
								atom.residue_id =	Integer.parseInt( line.substring(22,26).trim() );
							}
							catch (Exception ex){
								try{
									atom.residue_id =	Integer.parseInt(line.substring(23,27).trim() );
								}
								catch (Exception e){
									atom.residue_id = 1;
								}
							}
						}
						else{
							//try to get the residue number from differently placed fields
							String input = line.substring(22,27).trim();
							try{
								atom.residue_id =	Integer.parseInt(input);
							}
							catch (Exception ex){
								try{
									atom.residue_id =	Integer.parseInt( input );
								}
								catch (NumberFormatException e){
									//if input contains letters in addition to numbers, use only the numeric part
									try{
										Pattern pattern = Pattern.compile("[A-Za-z]");
										Matcher matcher = pattern.matcher(input);
										atom.residue_id = Integer.parseInt(matcher.replaceAll(""));
									}
									catch (NumberFormatException ee){
										atom.residue_id = 1;
									}
								}
							}
						}
						
						atom.coordinate = new double[3];
						atom.coordinate[0] =
							Double.parseDouble(line.substring(30,38).trim() );
						atom.coordinate[1] =
							Double.parseDouble(line.substring(38,46).trim() );
						atom.coordinate[2] =
							Double.parseDouble( line.substring(46,54).trim() );


						Vector records = (Vector) passComponents.get(
							StructureComponentRegistry.TYPE_ATOM );
						if ( records == null )
						{
							records = new Vector( );
							passComponents.put(
								StructureComponentRegistry.TYPE_ATOM, records );
						}
						records.add( atom );
//								System.out.println("added atom");
													
						try{
//									System.out.println("compound = " + atom.compound);
							if (ProteinProperties.isAminoAcid(atom.compound)){
								if (atom.residue_id != lastResidueId){
									//there was a switch of residue or this is only the start of the chain
									if (lastResidueName != null){
										//check if this is a chain change
										//also, this may be needed for gaps in residue numbering
										//to avoid making long bonds that span the gap
										if (lastC != null){
											dif = currentC.residue_id - lastC.residue_id;
										}
										if (lastChainId != null){
											if (!lastChainId.equals(currentChainId)){
												lastChainId = currentChainId;
												currentChainId = atom.chain_id;
												chainChange = true;
//														lastResidueName = null;
//														lastResidueId = -1;
											}
											else{
												currentChainId = atom.chain_id;
												chainChange = false;
											}
										}
										else{
											lastChainId = currentChainId;
											chainChange = false;
										}


										//there was a residue before
										//this is the first atom of the new residue, so process the
										//last residue for bonds and start up a new one
										HashMap map = ProteinProperties.getBondMapping(lastResidueName);
										if (map != null){
											//go over the atoms and create bonds
											Set keys = residueAtoms.keySet();
											Iterator it = keys.iterator();
											while (it.hasNext()){
												String name = (String)it.next();
												Atom a = (Atom)residueAtoms.get(name);
												
												if (name.equals("N") && lastC != null && !chainChange && dif <= 1){
													//create the bond with the last C
													if (Algebra.distance(lastC.coordinate, a.coordinate) > 2){
//																System.out.println("continue");
														continue;
													}
													Bond b = new Bond(lastC, a);
													b.setOrderOnly(1.0f);

													Vector r = (Vector) passComponents.get(
															StructureComponentRegistry.TYPE_BOND );
														if ( r == null )
														{
															r = new Vector( );
															passComponents.put(
																StructureComponentRegistry.TYPE_BOND, r );
														}
													r.add( b );
												}
												
												Iterator it2 = keys.iterator();
												while (it2.hasNext()){
													String nn = (String)it2.next();
													String label = a.name + ":" + nn;
													if (map.containsKey(label)){
														Bond b = new Bond(a, (Atom)residueAtoms.get(nn));
														float order = ((Float)map.get(label)).floatValue();
														if (order == 1.5f || order == 2.0f){
															a.hybridization = Atom.SP2;
															((Atom)residueAtoms.get(nn)).hybridization = Atom.SP2;
														}
														else if (order == 3.0f){
															a.hybridization = Atom.SP;
															((Atom)residueAtoms.get(nn)).hybridization = Atom.SP;
														}
														
														
														
														b.setOrderOnly(order);

														Vector r = (Vector) passComponents.get(
																StructureComponentRegistry.TYPE_BOND );
															if ( r == null )
															{
																r = new Vector( );
																passComponents.put(
																	StructureComponentRegistry.TYPE_BOND, r );
															}
														r.add( b );
													}
												}
												
												//make sure all N's except for Lys are sp2-hybridized
												if (a.element == 7){
													if (a.name.equals("NZ"))continue;//terminal N in lysine
													a.hybridization = Atom.SP2;
												}
											}
										}
										
									}
									else{
										//start of the chain: good place to put a one-time flag
										protein = true;
										dna = false;
										currentChainId = atom.chain_id;
										
									}
									residueAtoms.clear();
									
									lastResidueName = atom.compound;
									lastResidueId = atom.residue_id;
									
								}
								residueAtoms.put(atom.name, atom);
								if (atom.name.equals("C")){
									lastC = currentC;
									currentC = atom;
								}
								
							}
							
							if (DNAProperties.isNucleicAcid(atom.compound)){
								
								dna = true;
								
								//check for lastResidueId that preceded this one
								if (residueAtoms.size() > 0){
									
//											System.out.println("protein = " + protein);
//											System.out.println("dna = " + dna);
									
									if (protein){
										HashMap map = null;
										Set names = residueAtoms.keySet();
//												System.out.println("residueAtoms = " + residueAtoms.size());
										Iterator it = names.iterator();
										while (it.hasNext()){
											
											String name = (String)it.next();
											Atom a = (Atom)residueAtoms.get(name);
											if (a.element == 7){
												if (a.name.equals("NZ"))continue;
												a.hybridization = Atom.SP2;
											}

											if (map == null){
//														System.out.println("assigning map");
												map = ProteinProperties.getBondMapping(a.compound);
												if (map == null){
													//it could be dna
													map = DNAProperties.getBondMapping(a.compound);
												}
												
//														System.out.println("map = " + map);
											}
											
											
											//go over the atoms and create bonds
											
											if (a.name.equals("N")){
												//create the bond with the last C
												if (lastC != null && lastC.residue_id != a.residue_id){
													Bond b = new Bond(lastC, a);
													b.setOrderOnly(1.0f);
//															System.out.println("Created bond with the last C");
													Vector r = (Vector) passComponents.get(
															StructureComponentRegistry.TYPE_BOND );
														if ( r == null )
														{
															r = new Vector( );
															passComponents.put(
																StructureComponentRegistry.TYPE_BOND, r );
														}
													r.add( b );
												}
											}
														
											Iterator it2 = names.iterator();
											while (it2.hasNext()){
												String nn = (String)it2.next();
												String label = a.name + ":" + nn;
//														System.out.println("map = " + map + ", label = " + label);
//														System.out.println("a = " + a + ", nn = " + residueAtoms.get(nn));
												if (map != null && map.containsKey(label)){
													Bond b = new Bond(a, (Atom)residueAtoms.get(nn));
//															System.out.println("valid map");
													float order = ((Float)map.get(label)).floatValue();
													if (order == 1.5f || order == 2.0f){
														a.hybridization = Atom.SP2;
														((Atom)residueAtoms.get(nn)).hybridization = Atom.SP2;
													}
													else if (order == 3.0f){
														a.hybridization = Atom.SP;
														((Atom)residueAtoms.get(nn)).hybridization = Atom.SP;
													}
													
													b.setOrderOnly(order);
												
													Vector r = (Vector) passComponents.get( StructureComponentRegistry.TYPE_BOND );
													if ( r == null ){
														r = new Vector( );
														passComponents.put( StructureComponentRegistry.TYPE_BOND, r );
													}
													r.add( b );
//															System.out.println("added bond");
												}
											}
										}
									}
									
									protein = false;
								}

								if (atom.residue_id != lastResidueId){
									//there was a switch of residue or this is only the start of the chain
									if (lastResidueName != null){
										//check if this is a chain change
										//also, this may be needed for gaps in residue numbering
										//to avoid making long bonds that span the gap
										if (lastO != null){
											dif = currentO.residue_id - lastO.residue_id;
										}
										if (lastChainId != null){
											if (!lastChainId.equals(currentChainId)){
												lastChainId = currentChainId;
												currentChainId = atom.chain_id;
												chainChange = true;
//														lastResidueName = null;
//														lastResidueId = -1;
											}
											else{
												currentChainId = atom.chain_id;
												chainChange = false;
											}
										}
										else{
											lastChainId = currentChainId;
											chainChange = false;
										}


										//there was a residue before
										//this is the first atom of the new residue, so process the
										//last residue for bonds and start up a new one
										HashMap map = DNAProperties.getBondMapping(lastResidueName);
										if (map != null){
											//go over the atoms and create bonds
											Set keys = residueAtoms.keySet();
											Iterator it = keys.iterator();
											while (it.hasNext()){
												String name = (String)it.next();
												Atom a = (Atom)residueAtoms.get(name);
												if (a.element == 7){
//															System.out.println("SP2 for " + a.name);
													a.hybridization = Atom.SP2;
												}

												if (name.equals("P") && lastO != null && !chainChange && dif <= 1){
													//create the bond with the last C
													Bond b = new Bond(lastO, a);
													b.setOrderOnly(1.0f);

													Vector r = (Vector) passComponents.get(
															StructureComponentRegistry.TYPE_BOND );
														if ( r == null )
														{
															r = new Vector( );
															passComponents.put(
																StructureComponentRegistry.TYPE_BOND, r );
														}
													r.add( b );
												}
												
												Iterator it2 = keys.iterator();
												while (it2.hasNext()){
													String nn = (String)it2.next();
													String label = a.name + ":" + nn;
													if (map.containsKey(label)){
														Bond b = new Bond(a, (Atom)residueAtoms.get(nn));
														float order = ((Float)map.get(label)).floatValue();
														if (order == 1.5f || order == 2.0f){
															a.hybridization = Atom.SP2;
															((Atom)residueAtoms.get(nn)).hybridization = Atom.SP2;
														}
														//higher bond orders are not checked, since this is a standard set of amino
														//acids that don't have triple bonds
														
														b.setOrderOnly(order);

														Vector r = (Vector) passComponents.get(
																StructureComponentRegistry.TYPE_BOND );
															if ( r == null )
															{
																r = new Vector( );
																passComponents.put(
																	StructureComponentRegistry.TYPE_BOND, r );
															}
														r.add( b );
													}
												}
												
											}
										}
										
										
									}
									else{
										//start of the chain: good place to put a one-time flag
										protein = false;
										dna = true;
										currentChainId = atom.chain_id;
									}
									residueAtoms.clear();
									
									lastResidueName = atom.compound;
									lastResidueId = atom.residue_id;
									
								}
								residueAtoms.put(atom.name, atom);
								if (atom.name.equals("O3*") || atom.name.equals("O3'")){
									lastO = currentO;
									currentO = atom;
								}
								
							}
						
						}
						catch (Exception ex){
							ex.printStackTrace();
						}
						
						continue;
					}
		
					if (
							(line.startsWith("CONECT"))
						){
						
						if (conect == null){
							//initialize the conect map
							conect = new int[hetatmCount][hetatmCount];
							for (int j = 0; j < hetatmCount; j++){
								for (int k = 0; k < hetatmCount; k++){
									conect[j][k] = 0;
								}
							}
						}
						//parse out the bonding record
						try{
							String record = new String(line);
//									System.out.println("record = " + record);
							int n = record.indexOf("\n");
							int r = record.indexOf("\r");
							String template = null;
							if (n > 0 && r > 0){
								template = record.substring(6, Math.min(n, r));
							}
							else if (r > 0){
								template = record.substring(6, r);
							}
							else if (n > 0){
								template = record.substring(6, n);
							}
							else{
								template = record.substring(6);
							}
							StringTokenizer tok = new StringTokenizer(template, " ", false);
							int root = -1;//index of the root atom
							
							while (tok.hasMoreTokens()){
								try{
									int nn = Integer.parseInt(tok.nextToken());
									if (root == -1){
										//this is the root atom
										root = ((Integer)numberOrder.get(new Integer(nn))).intValue();
										continue;
									}
									conect[root][((Integer)numberOrder.get(new Integer(nn))).intValue()]++;
								}
								catch (Exception ex){
									continue;
								}

							}
						}
						catch (Exception ex){
							System.err.println("Erreur : " + ex);
							ex.printStackTrace();
						}
						
						
					}
					//
					// HELIX record
					//
					if (
						(line.startsWith("HELIX"))
					)
					{
						Helix helix = new Helix( );

						helix.name = line.substring(0,6).trim();
						helix.start_compound = line.substring(15,18).trim();
						helix.start_chain = line.substring(19,20).trim();
						if ( helix.start_chain == null ) helix.start_chain = "A";
						helix.start_residue = Integer.parseInt( line.substring(21, 25).trim() );
						helix.end_compound = line.substring(27,30).trim();
						helix.end_chain = line.substring(31,32).trim();
						if ( helix.end_chain == null ) helix.end_chain = "A";
						helix.end_residue = Integer.parseInt(line.substring(33,37).trim() );
						helix.righthand = true; // JLM DEBUG

						Vector records = (Vector) passComponents.get(
							StructureComponentRegistry.TYPE_HELIX );
						if ( records == null )
						{
							records = new Vector( );
							passComponents.put(
								StructureComponentRegistry.TYPE_HELIX, records );
						}
						records.add( helix );

						continue;
					}

					//
					// TURN record
					//
					if (
						(line.startsWith("TURN"))
					)
					{
						Turn turn = new Turn( );

						turn.name = line.substring(0,6).trim();
						turn.start_compound = line.substring(15,18).trim();
						turn.start_chain = line.substring(19,20).trim();
						if ( turn.start_chain == null ) turn.start_chain = "A";
						turn.start_residue = Integer.parseInt(line.substring(20,24).trim() );
						turn.end_compound = line.substring(26,29).trim();
						turn.end_chain = line.substring(30,31).trim();
						if ( turn.end_chain == null ) turn.end_chain = "A";
						turn.end_residue = Integer.parseInt( line.substring(31,35).trim() );

						Vector records = (Vector) passComponents.get(
							StructureComponentRegistry.TYPE_TURN );
						if ( records == null )
						{
							records = new Vector( );
							passComponents.put(
								StructureComponentRegistry.TYPE_TURN, records );
						}
						records.add( turn );

						continue;
					}

					//
					// STRAND (SHEET) record
					//
					if (
						(line.startsWith("SHEET"))
					)
					{
						Strand strand = new Strand( );

						// Each individual SHEET record is really a single STRAND.
						// And it takes multiple SHEET records in order to
						// fully specify a sheet structure.
						strand.name = line.substring(0,6).trim();
						strand.start_compound = line.substring(17,20).trim();
						strand.start_chain = line.substring(21,22).trim();
						if ( strand.start_chain == null ) strand.start_chain = "A";
						strand.start_residue = Integer.parseInt( line.substring(22,26).trim() );
						strand.end_compound = line.substring(28,31).trim();
						strand.end_chain = line.substring(32,33).trim();
						if ( strand.end_chain == null ) strand.end_chain = "A";
						strand.end_residue = Integer.parseInt( line.substring(33,37).trim() );

						Vector records = (Vector) passComponents.get(
							StructureComponentRegistry.TYPE_STRAND );
						if ( records == null )
						{
							records = new Vector( );
							passComponents.put(
								StructureComponentRegistry.TYPE_STRAND, records );
						}
						records.add( strand );
						continue;
					}

				}
				
				//create bonds in the last residue that never was reached (for proteins and DNA only)
				if (residueAtoms.size() > 0){
					
//					System.out.println("protein = " + protein);
//					System.out.println("dna = " + dna);
					
					if (protein){
						HashMap map = null;
						Set names = residueAtoms.keySet();
						Iterator it = names.iterator();
						while (it.hasNext()){
							
							String name = (String)it.next();
							Atom a = (Atom)residueAtoms.get(name);
							if (map == null){
//								System.out.println("assigning map");
								map = ProteinProperties.getBondMapping(a.compound);
								if (map == null){
									//it could be dna
									map = DNAProperties.getBondMapping(a.compound);
								}
								
//								System.out.println("map = " + map);
							}
							//go over the atoms and create bonds
							
							if (a.name.equals("N")){
								//create the bond with the last C
								if (lastC != null && lastC.residue_id != a.residue_id){
									Bond b = new Bond(lastC, a);
									b.setOrderOnly(1.0f);
		
									Vector r = (Vector) passComponents.get(
											StructureComponentRegistry.TYPE_BOND );
										if ( r == null )
										{
											r = new Vector( );
											passComponents.put(
												StructureComponentRegistry.TYPE_BOND, r );
										}
									r.add( b );
								}
							}
										
							Iterator it2 = names.iterator();
							while (it2.hasNext()){
								String nn = (String)it2.next();
								String label = a.name + ":" + nn;
//								System.out.println("map = " + map + ", label = " + label);
//								System.out.println("a = " + a + ", nn = " + residueAtoms.get(nn));
								if (map != null && map.containsKey(label)){
									Bond b = new Bond(a, (Atom)residueAtoms.get(nn));
//									System.out.println("valid map");
									float order = ((Float)map.get(label)).floatValue();
									if (order == 1.5f || order == 2.0f){
										a.hybridization = Atom.SP2;
										((Atom)residueAtoms.get(nn)).hybridization = Atom.SP2;
									}
									else if (order == 3.0f){
										a.hybridization = Atom.SP;
										((Atom)residueAtoms.get(nn)).hybridization = Atom.SP;
									}
									
									b.setOrderOnly(order);
								
									Vector r = (Vector) passComponents.get( StructureComponentRegistry.TYPE_BOND );
									if ( r == null ){
										r = new Vector( );
										passComponents.put( StructureComponentRegistry.TYPE_BOND, r );
									}
									r.add( b );
//									System.out.println("added bond");
								}
							}
							if (a.element == 7){
								if (a.name.equals("NZ"))continue;
								a.hybridization = Atom.SP2;
							}
						}
						
						protein = false;
					}
				
				
					if (dna){
						HashMap map = null;
						Set names = residueAtoms.keySet();
						Iterator it = names.iterator();
						while (it.hasNext()){
							
							String name = (String)it.next();
							Atom a = (Atom)residueAtoms.get(name);
							if (map == null){
								map = DNAProperties.getBondMapping(a.compound);
								if (map == null){
									map = ProteinProperties.getBondMapping(a.compound);
								}
							}
							//go over the atoms and create bonds
							
							if (a.name.equals("P")){
								//create the bond with the last O
								if (lastO != null && lastO.residue_id != a.residue_id){
									Bond b = new Bond(lastO, a);
									b.setOrderOnly(1.0f);
		
									Vector r = (Vector) passComponents.get(
											StructureComponentRegistry.TYPE_BOND );
										if ( r == null )
										{
											r = new Vector( );
											passComponents.put(
												StructureComponentRegistry.TYPE_BOND, r );
										}
									r.add( b );
								}
							}
										
							Iterator it2 = names.iterator();
							while (it2.hasNext()){
								String nn = (String)it2.next();
								String label = a.name + ":" + nn;
								if (map != null && map.containsKey(label)){
									Bond b = new Bond(a, (Atom)residueAtoms.get(nn));
									float order = ((Float)map.get(label)).floatValue();
									if (order == 1.5f || order == 2.0f){
										a.hybridization = Atom.SP2;
										((Atom)residueAtoms.get(nn)).hybridization = Atom.SP2;
									}
									
									b.setOrderOnly(order);
								
									Vector r = (Vector) passComponents.get(
											StructureComponentRegistry.TYPE_BOND );
										if ( r == null )
										{
											r = new Vector( );
											passComponents.put(
												StructureComponentRegistry.TYPE_BOND, r );
										}
									r.add( b );
								}
							}
							
							//make sure all N's except for Lys are sp2-hybridized
							if (a.element == 7){
								a.hybridization = Atom.SP2;
							}
						}
						
						dna = false;
					}
					
				}
				
				
				//go through the conect records, if any, and create bonds
				if (hetatmCount > 0 && conect != null){
					for (int i = 0; i < hetatmCount; i++){
						for (int j = i; j < hetatmCount; j++){
							if (i == j) continue;
							//get the real atom numbers
							int order = conect[i][j];
							if (order == 0) continue;
							
							Atom a1 = (Atom)hetatms.get(i);
							Atom a2 = (Atom)hetatms.get(j);
							
							//create a bond
							Bond bond = new Bond(a1, a2);
							if (order == 1){
								bond.setOrderOnly(1.0f);//the default is already SP3 and atoms with another value should not be touched
							}
							else if (order == 2){
								bond.setOrderOnly(2.0f);
								a1.hybridization = Atom.SP2;
								a2.hybridization = Atom.SP2;
							}
							else if (order == 3){
								bond.setOrderOnly(3.0f);
								a1.hybridization = Atom.SP;
								a2.hybridization = Atom.SP;
							}
							
							Vector records = (Vector) passComponents.get(
									StructureComponentRegistry.TYPE_BOND );
								if ( records == null )
								{
									records = new Vector( );
									passComponents.put(
										StructureComponentRegistry.TYPE_BOND, records );
								}
							records.add( bond );

						}
					}
				}
				
			}
			catch (NumberFormatException ex){
				ex.printStackTrace();
				if (Status.enabled) Status.progress( 1.0f, null );
				throw ex;
			}
			catch (Exception ex){
				ex.printStackTrace();
				return null;
			}
			
			//IMPORTANT: if chains are renamed, also scan conformational records to make sure they have correct chain ids
			//use their start and end residues
			
			//check ter records
			if (ter.size() > 0){
				//check for chain ids
				Vector records = (Vector) passComponents.get(StructureComponentRegistry.TYPE_ATOM );
				if ( records != null ){
					for (int i = 0; i < records.size(); i++){
						Atom a = (Atom)records.get(i);
						if (a.chain_id.equals("_")){
							//check the last atom to see if there was an id
							if (i >= 1){
								Atom last = (Atom)records.get(i-1);
								if (ter.contains(last)){
									//we have just started a new chain
									String ci = last.chain_id;
									
									//it can't be "_", since we have just passed over it, so figure out what letter it has
									//it can be just an increment for the replacements, or a real id that was there from the start
									int lastIndex = -1;
									for (int j = 0; j < chainIdList.length; j++){
										if (ci.equals(chainIdList[j])){
											chainIdCounter = j+1;
											break;
										}
									}
								}
								
								//it may be necessary to switch to the next chain
							}
							
							//reset it to a lettered chain id
							a.chain_id = chainIdList[chainIdCounter];
						}
						
					}
				}
				
			}
			
			//
			// Create the Structure object
			//
			Structure structure = new Structure()
			{
				// A hashtable of vectors where
				// each hash KEY is the StructureComponent type String.
				// each hash VALUE is a Vector of StructureComponent objects.
				protected Hashtable structureComponents = null;

				// To free up the global state for another load call.
				private String localUrlString;

				// public Structure()  Anonymous inner class constructor.
				{
					structureComponents = passComponents;
					passComponents = null;

					localUrlString = urlString;
					
				}

				public String getUrlString( ){
					return localUrlString;
				}

				
				public int getStructureComponentCount( String scType ){
					Vector records = (Vector) structureComponents.get( scType );
					if ( records == null )
						return 0;
					else
						return records.size( );
				}

				private void getStructureComponentByIndex( int index,
					StructureComponent structureComponent )
					throws IndexOutOfBoundsException, IllegalArgumentException
				{
					String type = structureComponent.getStructureComponentType();
					if ( type == null )
						throw new IllegalArgumentException( "type inconnu" + type );

					Vector records = (Vector) structureComponents.get( type );
					if ( records == null )
						throw new IllegalArgumentException( "pas d\'enregistrement du type " + type );

					StructureComponent cachedComponent =
						(StructureComponent) records.elementAt( index );

					cachedComponent.structure = this;
					structureComponent.structure = this;
					structureComponent.copy( cachedComponent );
	            }

				public StructureComponent getStructureComponentByIndex( String type,
					int index )
					throws IndexOutOfBoundsException, IllegalArgumentException
				{
					Vector records = (Vector) structureComponents.get( type );
					if ( records == null )
						throw new IllegalArgumentException( "pas d\'enregistrement du type " + type );

					StructureComponent structureComponent =
						(StructureComponent) records.elementAt( index );
					structureComponent.structure = this;

					return structureComponent;
	            }
			};

			// Progress is done.
			Status.progress( 1.0f, null );

			return structure;
		}
	
    public static byte[] getCompressedData(byte[] input){
//   	 Create the compressor with highest level of compression
       Deflater compressor = new Deflater();
       compressor.setLevel(Deflater.BEST_COMPRESSION);
       
       // Give the compressor the data to compress
       compressor.setInput(input);
       compressor.finish();
       
       // Create an expandable byte array to hold the compressed data.
       // You cannot use an array that's the same size as the orginal because
       // there is no guarantee that the compressed data will be smaller than
       // the uncompressed data.
       ByteArrayOutputStream bos = new ByteArrayOutputStream(input.length);
       
       // Compress the data
       byte[] buf = new byte[1024];
       while (!compressor.finished()) {
           int count = compressor.deflate(buf);
           bos.write(buf, 0, count);
       }
       try {
           bos.close();
       } catch (IOException e) {
       }
       
       // Get the compressed data
       byte[] compressedData = bos.toByteArray();
       
       return compressedData;
   }
	
   public static byte[] getDecompressedData(byte[] input){
   	
//   	 Create the decompressor and give it the data to compress
       Inflater decompressor = new Inflater();
       decompressor.setInput(input);
       
       // Create an expandable byte array to hold the decompressed data
       ByteArrayOutputStream bos = new ByteArrayOutputStream(input.length);
       
       // Decompress the data
       byte[] buf = new byte[1024];
       while (!decompressor.finished()) {
           try {
               int count = decompressor.inflate(buf);
               bos.write(buf, 0, count);
           } catch (DataFormatException e) {
           }
       }
       try {
           bos.close();
       } catch (IOException e) {
       }
       
       // Get the decompressed data
       byte[] decompressedData = bos.toByteArray();
       
       return decompressedData;
   }


}
