package edu.sdsc.sirius.md;

// MBT
import edu.sdsc.mbt.image.*;

import edu.sdsc.sirius.util.*;
import edu.sdsc.sirius.util.PagePrintable;

// Core
import java.util.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.filechooser.FileFilter;

import java.awt.image.*;
import java.io.*;
import java.awt.print.*;

import javax.imageio.*;
import javax.imageio.spi.*;
import javax.imageio.stream.*;

public class TemperaturePanel extends JDialog {

	private HashMap data;
	private JPanel imagePanel = new JPanel();
	private JPanel markerPanel = new JPanel();
	
	private JDialog panelFrame;
	private JFrame parent;
	private MDPanel mdPanel;
	
	private BufferedImage marker_image;
	private Graphics2D g;

	private BufferedImage m_image;
	private Graphics2D g2;
		
	private int imageWidth = 500;
	private int imageHeight = 380;
	private int leftMargin = 40;
	private int bottomMargin = 50;
	private int topMargin = 20;
	private int rightMargin = 50;
	
	private JLayeredPane pane;
	
	private Object value = new Object();//as a placeholder value for hashes
	
	private MouseInputAdapter mia1;
	private MouseInputAdapter mia2;
	protected static int counter1 = 0;
	protected static int counter2 = 0;
	
	private Book book = null;
	public PageFormat mPageFormat;
	protected int m_maxNumPage = 1;
	
	private double[] timescales = { 0.001, 0.001, 0.01, 0.1, 0.25, 0.5, 1.0, 2.0, 5.0, 10.0, 25.0, 50.0, 100.0, 250.0, 500.0, 1000.0, 5000.0, 10000.0 };
	
	private Manager callable;
	private JPopupMenu imagePopup;
	
	private double startTime = 0;
	private double endTime = 0;
	
	private double minTemp = 1000;
	private double maxTemp = 0;//scale height is 300, so 2K/pixel
	
	private Vector dataFrames;
	
	private int pixelStep = 0;//how many pixels per frame advance
	
	private Image marker;
	private JLabel markerLabel;
	
	private final int colors = 239;
	
	private int currentMarkerPosition = leftMargin;
	
	private int scaleWidth = 0;
	private int scaleHeight = 0;
	
	public TemperaturePanel(Manager callable, MDPanel mdPanel, Vector dataFrames){
	
		super((JDialog)mdPanel, "Temprature du systme", false);
		parent = callable.getApplicationFrame();
		this.mdPanel = mdPanel;
		
		this.callable = callable;
		this.dataFrames = dataFrames;
		
		//create the image and place it on the image panel
		imagePanel.setBounds(5,0,500,410);
		markerPanel.setBounds(5,0,500,410);
		markerPanel.setOpaque(false);
		
		pane = new JLayeredPane();
		
		//analyze input data to get the basic factors
		for (int i = 0; i < dataFrames.size(); i++){
			double[] frame = (double[])dataFrames.get(i);
			if (i == 0){
				startTime = frame[1];
			}
			if (i == dataFrames.size()-1){
				endTime = frame[1];
			}
			
			if (frame[2] < minTemp){
				minTemp = frame[2];//these will be used to format the vertical scale
			}
			if (frame[2] > maxTemp){
				maxTemp = frame[2];
			}
			
		}
		
		
		Image graph = paintImage();
		ImageIcon icon = new ImageIcon(graph);
		JLabel image = new JLabel(icon);
		imagePanel.add(image);
				
		marker = paintMarker();
		ImageIcon icon2 = new ImageIcon(marker);
		markerLabel = new JLabel(icon2);
		markerPanel.add(markerLabel);

		panelFrame = this;
		
		imagePopup = createImagePopup();
		
		mia1 = new MouseInputAdapter(){
		
			boolean popup = false;
			
			public void mousePressed(MouseEvent e){
				setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
				checkPopup(e);
			}

			public void mouseReleased(MouseEvent e){
				checkPopup(e);
			}
			
			public void mouseClicked(MouseEvent e){
				checkPopup(e);
			}

			public void checkPopup(MouseEvent e){
				if (e.isPopupTrigger()){
					popup = true;
				}
				start(e, popup);
			}
			
			public void start(MouseEvent e, boolean p){
				counter1++;
				if (counter1 == 3){
					counter1 = 0;
					if (popup){
						imagePopup.show(imagePanel, e.getX(), e.getY());
//						System.out.println("popup");
					}
					else{
//						System.out.println("Processing click");
					}
					popup = false;
				}
				else{
//					System.out.println("counter = " + counter);
				}
			}
			
		};
		image.addMouseListener( mia1 );
		image.addMouseMotionListener( mia1 );

		pane.setBounds(0,0,510, 385);
		pane.setPreferredSize(new Dimension(510,415));
		pane.add(imagePanel, new Integer(10));
		pane.add(markerPanel, new Integer(20));
		pane.setVisible(true);
		
		getContentPane().add(pane);
		final MDPanel mp = mdPanel;
		
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				mp.closeTempPanel(true);
			}
		});

		//printing stuff
		PrinterJob pj = PrinterJob.getPrinterJob();
		mPageFormat = pj.defaultPage();
		mPageFormat.setOrientation(PageFormat.LANDSCAPE);
		pj = null;

		//position it right below the main md panel
		Rectangle r = mdPanel.getBounds();
		RMSDPanel e = mdPanel.getRMSDPanel();
		if (e != null && e.isVisible()){
			Rectangle er = e.getBounds();
			setBounds(r.x + 100, r.y+r.height+100,515, 415);
		}
		else{
			setBounds(r.x, r.y+r.height,515, 415);
		}
		setResizable(false);
		setVisible(true);

	}
	
	public void closePanel(){
		setVisible(false);
		dispose();
	}
	
	public Image paintImage(){
	
		//use the data HashMap to create the image
		m_image = new BufferedImage(imageWidth, imageHeight, BufferedImage.TYPE_BYTE_INDEXED, createColorModel());
		g2 = m_image.createGraphics();

		scaleWidth = imageWidth - leftMargin - rightMargin;
		scaleHeight = imageHeight - 80;
		
		double timespan = endTime - startTime;
		//figure out what time scale would be best
		
		
			
		pixelStep = (int)(scaleWidth/(dataFrames.size()-1));
		
		//set margins at 15 pixels, so the total left for the graph itself is scaleHeight-30
		
		int graphHeight = scaleHeight - 30;
		
		//draw the background
		g2.setColor(Color.white);
		g2.fillRect(0,0,imageWidth, imageHeight);
		g2.setColor(Color.black);
		g2.drawRect(0,0,imageWidth - 1, imageHeight - 1);
		
		//draw the axes
		g2.drawLine(leftMargin, topMargin, leftMargin, topMargin + scaleHeight);
		g2.drawLine(leftMargin, topMargin + scaleHeight, leftMargin + scaleWidth, topMargin + scaleHeight);
		
		//we can draw time scale using ps
		//read the last frame to get the final timestamp
//		System.out.println("time = " + time);
		
		double ratio = (endTime - startTime)/scaleWidth;
		
		double interval = 0;
		for (int i = timescales.length-1; i >= 0; i--){
			double sratio = (endTime - startTime)/timescales[i];
			if (sratio > 5){
				interval = timescales[i];
				break;
			}
		}
		
		int ir = (int)(interval/ratio);
		
		g2.setFont(new Font("Dialogue", Font.PLAIN, 9));

		
		g2.drawString(StringUtils.getFormattedNumber(startTime, 1), leftMargin-5, topMargin+scaleHeight+20);

		//draw the scale
		int sx = leftMargin + ((int)(interval/ratio));
		int sy = topMargin+scaleHeight;
		int counter = 1;
		int limit = leftMargin + scaleWidth;
		
		
		while (true){
			if (sx > limit) break;
			g2.drawLine(sx, topMargin, sx, sy+5);
			double vv = startTime + interval*counter;
			String mark = null;
			if (interval > 1){
				mark = StringUtils.getFormattedNumber(vv, 0);
			}
			else if (interval > 0.05){
				mark = StringUtils.getFormattedNumber(vv, 1);
			}
			else if (interval < 0.01){
				mark = StringUtils.getFormattedNumber(vv, 3);
			}
			else{
				mark = StringUtils.getFormattedNumber(vv, 2);
			}
			g2.drawString(mark, sx-5, sy+20);
			
			sx += ir;
			counter++;
		}
		
		g2.setFont(new Font("Dialogue", Font.PLAIN, 10));

		
		//draw the ps string 
		g2.drawString("ps", leftMargin+scaleWidth+25, topMargin+scaleHeight+20);
		
		//figure out base temperature level on the graph X axis, plus number of T units per pixel
		double tempSpan = maxTemp - minTemp;
		double totalTempSpan = tempSpan*1.3;//to have extra space on both ends
		
//		System.out.println("tempSpan = " + tempSpan);
//		System.out.println("totalTempSpan = " + totalTempSpan);
		
		double baseTemp = 0;//temperature at the X axis
		if (minTemp > 0){
			baseTemp = minTemp - tempSpan*0.15;
		}
		
//		System.out.println("baseTemp = " + baseTemp);
		
		double TempRatio = totalTempSpan/scaleHeight;//how many temperature units per vertical pixel
//		System.out.println("tempRatio = " + TempRatio);
		
		int zerolevel = topMargin + scaleHeight;
		
//		System.out.println("zerolevel = " + zerolevel);
		
		//draw the zero line
		g2.drawString("Temp, K", 5, 15);

//		System.out.println("zerolevel = " + zerolevel + ", pixelStep = " + pixelStep);
		
//		System.out.println("energy: dataFrames = " + dataFrames.size());
		
		
		
		//draw the graph itself
		Point lastTemp = new Point(-1,-1);
//		g2.setStroke(new BasicStroke(1.5f));
		if (pixelStep > 0){
			for (int i = 0; i < dataFrames.size(); i++){
				double[] frame = (double[])dataFrames.get(i);
				int x = leftMargin + i*pixelStep;
				
				int yk = zerolevel - (int)((frame[2] - baseTemp)/TempRatio);
				if (lastTemp.x > -1){
					g2.setColor(Color.BLACK);
					g2.drawLine(lastTemp.x,lastTemp.y, x, yk);
				}
				
				lastTemp.x = x;
				lastTemp.y = yk;
	

			}
		}
		else{
			double stepRatio = (double)dataFrames.size()/(double)scaleWidth;
			//number of steps is larger than number of available pixels, so steps must be skipped
			//in this case, walk along pixels (instead of values), and compute the corresponding value for the given pixel
			Point lastValue = new Point(-1,-1);
			for (int i = 0; i < scaleWidth; i++){
				int currentFrame = (int)Math.round(stepRatio*i);
				
				double[] frame = (double[])dataFrames.get(currentFrame);
				int x = leftMargin + i;
				int yt = zerolevel - (int)((frame[2] - baseTemp)/TempRatio);
				
				if (lastTemp.x > -1){
					g2.setColor(Color.BLACK);
					g2.drawLine(lastTemp.x,lastTemp.y, x, yt);
				}
				
				lastTemp.x = x;
				lastTemp.y = yt;
				
			}
		}

		
		g2.setColor(Color.BLACK);		
		g2.setFont(new Font("Dialogue", Font.PLAIN, 10));
		g2.setStroke(new BasicStroke(1.0f));
		//draw in both directions from zero
		
		int tempInterval = (int)(totalTempSpan/7);
		
		sx = leftMargin;
		sy = zerolevel;
		for (int i = 0; i <= 5; i++){
			int value = (int)(i*tempInterval + baseTemp);
			String tag = new Integer(value).toString();
			g2.drawLine(sx, sy, sx-5, sy);
			g2.drawString(tag, 5, sy+5);
			sy -= 50;
		}
		
		return m_image;
	}
	
	public Image paintMarker(){
		
		//use the data HashMap to create the image
		marker_image = new BufferedImage(imageWidth, imageHeight, BufferedImage.TYPE_BYTE_INDEXED, createColorModel());
		g = marker_image.createGraphics();
		
		return marker_image;
	}
	
	public void updateMarker(int frame){
		
		if (frame < 1){
			if (g != null){
				g.setComposite( AlphaComposite.getInstance( AlphaComposite.SRC, 0.0f ) );
				g.setColor(Color.white);
				g.drawLine(currentMarkerPosition, topMargin, currentMarkerPosition, topMargin+scaleHeight);
				markerPanel.repaint();
			}
			return;
		}

		if (markerPanel == null){
			try{
				markerPanel.remove(markerLabel);
			}
			catch (Exception e){}
			marker = paintMarker();
			ImageIcon icon2 = new ImageIcon(marker);
			markerLabel = new JLabel(icon2);
			markerPanel.add(markerLabel);
			currentMarkerPosition = leftMargin;
			markerPanel.revalidate();
			markerPanel.repaint();
		}
		else{
			//simply update the marker line
			//draw over the last line
			g.setComposite( AlphaComposite.getInstance( AlphaComposite.SRC, 0.0f ) );
			g.setColor(Color.white);
			g.drawLine(currentMarkerPosition, topMargin, currentMarkerPosition, topMargin+scaleHeight);
			
			
			int x = 0;
			
			if (pixelStep > 0){
				x = leftMargin + (frame-1)*pixelStep;
			}
			else{
				double stepRatio = (double)dataFrames.size()/(double)scaleWidth;
//				System.out.println("stepRatio = " + stepRatio);
				x = leftMargin + (int)((frame-1)/stepRatio);
			}
//			System.out.println("marker at " + x);
//			g.setXORMode(Color.BLACK);
			g.setComposite( AlphaComposite.getInstance( AlphaComposite.SRC, 1.0f ) );
			g.setColor(Color.red);
			g.drawLine(x, topMargin, x, topMargin+scaleHeight);
			currentMarkerPosition = x;
			
//			g.setPaintMode();
			
			markerPanel.repaint();
		}
		
	}

	
	public JPopupMenu createImagePopup(){
	
		JPopupMenu popup = new JPopupMenu();
		
		JMenuItem blank = new JMenuItem("");
		blank.setEnabled(false);

		JMenuItem saveData = new JMenuItem("Sauvegarder des donnes de temprature...");
		saveData.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				saveData();
				repaint();
			}
				
		});

		JMenuItem save = new JMenuItem("Enregistrer l\'image...");
		save.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				saveImage();
				repaint();
			}
				
		});
		
		
		JMenuItem printSetup = new JMenuItem("Initialisation de l\'impression...");
		printSetup.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				printSetupHandler();
			}
		});
		JMenuItem print = new JMenuItem("Imprimer l\'image...");
		print.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				printImage();
			}
		});
		
		JMenuItem close = new JMenuItem("Fermer");
		close.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				mdPanel.closeTempPanel(true);
			}
		});
		
		popup.add(blank);
		popup.add(saveData);
		popup.add(save);
		popup.add(new JSeparator());
		popup.add(printSetup);
		popup.add(print);
		popup.add(new JSeparator());
		popup.add(close);
		 
		return popup;
	}
		
	public void saveImage(){

		try{
			ImageHandler.saveImage(callable, m_image);
		}
		catch (IOException e){
			JOptionPane.showMessageDialog(this, "Erreur d\'criture du fichier :" + e, "Erreur", JOptionPane.ERROR_MESSAGE);
		}

	}
	
	public void saveData(){

		if (dataFrames == null || dataFrames.size() == 0) return;
		
		//get the desired file name
		JFileChooser chooser = new JFileChooser(callable.getLastUsedDirectory());
		chooser.setDialogTitle("Sauvegarde de donnes de temprature");

		chooser.setDialogType(JFileChooser.SAVE_DIALOG);

		chooser.addChoosableFileFilter( new FileNameFilter("txt", "Format texte dlimit par des espaces") );
		chooser.addChoosableFileFilter( new FileNameFilter("txt", "Format texte dlimit par des Tab") );
		chooser.addChoosableFileFilter( new FileNameFilter("csv", "Format CSV sparations par des virgules") );
		
		chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		int result = chooser.showSaveDialog(callable.getApplicationFrame());
		if (result == JFileChooser.CANCEL_OPTION){
			return;
		}
		
		File filename = chooser.getSelectedFile();
		
		callable.setLastUsedDirectory(filename.getParent());
		
		String fname = filename.toString();
		
		int dot = fname.lastIndexOf(".");

		//get the desired file type
		FileFilter filter = chooser.getFileFilter();
		
		if (dot <= 0){
			//no extension supplied
			fname += "." + filter.toString();
		}

		StringBuffer buffer = new StringBuffer();
		if (filter.getDescription().startsWith("Espace")){
			buffer.append("Frame Time(ps) Temperature\n");
			for (int i = 1; i <= dataFrames.size(); i++){
				buffer.append(new Integer(i).toString());
				buffer.append(" ");
				double[] v = (double[])dataFrames.get(i-1);
				buffer.append(StringUtils.getFormattedNumber(v[1], 4));
				buffer.append(" ");
				buffer.append(StringUtils.getFormattedNumber(v[2], 3));
				buffer.append("\n");
			}
		}
		else if (filter.getDescription().startsWith("Tab")){
			buffer.append("Frame\ttime(ps)\tTemperature\n");
			for (int i = 1; i <= dataFrames.size(); i++){
				buffer.append(new Integer(i).toString());
				buffer.append("\t");
				double[] v = (double[])dataFrames.get(i-1);
				buffer.append(StringUtils.getFormattedNumber(v[1], 4));
				buffer.append("\t");
				buffer.append(StringUtils.getFormattedNumber(v[2], 4));
				buffer.append("\n");
			}
		}
		else if (filter.getDescription().startsWith("Virgule")){
			buffer.append("Frame,time(ps),Temperature\n");
			for (int i = 1; i <= dataFrames.size(); i++){
				buffer.append(new Integer(i).toString());
				buffer.append(",");
				double[] v = (double[])dataFrames.get(i-1);
				buffer.append(StringUtils.getFormattedNumber(v[1], 4));
				buffer.append(",");
				buffer.append(StringUtils.getFormattedNumber(v[2], 4));
				buffer.append("\n");
			}
		}
		else{
			callable.displayErrorMessage("Option de format de fichier illgale");
			return;
		}
		
		try{
			FileWriter fr = new FileWriter(fname);
			PrintWriter printer = new PrintWriter(fr);
			printer.print(buffer.toString());
			printer.close();
		}
		catch (Exception e){
			e.printStackTrace();
			
		}
	}
	
	public void printImage(){
		
		//print the currently displayed image
		
		Thread t = new Thread(){
			public void run(){

				PrinterJob pj = PrinterJob.getPrinterJob();

				//set the cursor, 'cause the wait will be long
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

				book = new Book();
				
				//get image dimensions
				int iWidth = m_image.getWidth();
				int iHeight = m_image.getHeight();
				double aspectRatio = imageWidth/imageHeight;
				
				//get the page format
				int pageWidth = (int)mPageFormat.getImageableWidth();
				int pageHeight = (int)(mPageFormat.getImageableHeight()*0.9);//make it a bit more legible by compression


				//now with this image, create a Printable instance and add it to Book
				PagePrintable p = new PagePrintable(m_image, pageWidth, pageHeight, iWidth, iHeight);

				book.append(p, mPageFormat);




				pj.setPageable(book);
				if (pj.printDialog()){
					try{
						pj.print();
					}
					catch (PrinterException pe){
						System.out.println("Erreur " + pe);
						JOptionPane.showMessageDialog(panelFrame, "Erreur d\'impression" + pe, "Erreur", JOptionPane.ERROR_MESSAGE);
					}
				}

				setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
				book = null;
			}
		};
		
		t.start();
		
		
	}			
	
	
	public void printSetupHandler(){

		//this method is called from Viewer when Print Setup is selected
		//currently active frame is found and this method is called

		Thread t = new Thread(){
			public void run(){
				PrinterJob pj = PrinterJob.getPrinterJob();
				mPageFormat = pj.pageDialog(mPageFormat);
			}
		};
		t.start();
	}
	
	
	private void savePrint( BufferedImage image, String filename ){

		try
		{
			File file = new File("D:\\print" + filename + ".png");
			// Get the currently-selected item in the combo box
			FormatDescriptor fd =
				(FormatDescriptor) (getWriterFormats().elementAt(2));

			// Get the ImageWriter stored in this FormatDescriptor
			ImageWriter imageWriter = fd.imageWriter;

			// Get an output stream for the file
			FileOutputStream fout = new FileOutputStream( file );

			// Turn it into an ImageOutputStreamn
			ImageOutputStream ios = ImageIO.createImageOutputStream( fout );

			// Plug this stream into the ImageWriter
			imageWriter.setOutput( ios );

			// Write the image
			imageWriter.write( image );

			// Dont forget to close the file!
			fout.close();
		}
		catch( IOException ie )
		{
			System.out.println("Impossible d\'enregistrer l\'image");
			ie.printStackTrace( );
		}
	}
	
	static private Vector getWriterFormats( )
	{
		Vector formats = new Vector( );
		Hashtable seen = new Hashtable( );

		String names[] = ImageIO.getWriterFormatNames( );
		for ( int i=0; i<names.length; ++i )
		{
			String name = names[i];
			Iterator writers = ImageIO.getImageWritersByFormatName( name );
			while ( writers.hasNext() )
			{
				ImageWriter iw = (ImageWriter) writers.next( );
				ImageWriterSpi iws = iw.getOriginatingProvider( );
				String suffixes[] = iws.getFileSuffixes( );
				boolean allseen = true;
				for ( int j=0; j<suffixes.length; ++j )
				{
					String suffix = suffixes[j];
					suffix = suffix.toLowerCase( );
					if ( ! seen.containsKey( suffix ) )
					{
						seen.put( suffix, suffix );
						String description = name + " (*." + suffix + ")";
						FormatDescriptor fd =
							new FormatDescriptor( suffix, iw, description );
						formats.addElement( fd );
					}
				}
			}
		}
	

		return formats;
	}

		
	static private class FormatDescriptor
	{
		public String suffix;
		public ImageWriter imageWriter;
		public String description;

		public FormatDescriptor( String suffix, ImageWriter imageWriter,
			String description )
		{
			this.suffix = suffix;
			this.imageWriter = imageWriter;
			this.description = description;
		}

		public String toString( )
		{
			return description;
		}
	}
	
	public IndexColorModel createColorModel() {

		byte[] red   = { (byte)255,(byte)255,0,0,(byte)255,0,(byte)255,0,(byte)240,25,(byte)232,(byte)155,20,(byte)161,43,(byte)229,(byte)229,20,(byte)252,5,(byte)255,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,0,25 };
		byte[] green  = { (byte)255,0,(byte)255,0,0,(byte)255,(byte)255,100,(byte)240,15,123,(byte)155,50,43,(byte)161,(byte)161,43,(byte)140,93,5,(byte)255,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)50,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)70,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)90,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)110,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)130,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,(byte)150,0,25 };
		byte[] blue = { (byte)255,0,0,(byte)255,(byte)255,(byte)255,0,0,25,100,20,(byte)155,(byte)170,(byte)229,(byte)229,43,(byte)161,20,100,5,(byte)255,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,(byte)50,(byte)70,(byte)90,(byte)110,(byte)130,(byte)150,0,25 };
		byte[] alpha = { 0,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255,(byte)255};

		IndexColorModel cm = new IndexColorModel(8, colors, red, green, blue, alpha);

		red = null;
		green = null;
		blue = null;
		alpha = null;

		return cm;
	}

	
}


