package edu.sdsc.sirius.search;

import edu.sdsc.mbt.util.*;
import edu.sdsc.mbt.viewables.StylesPreferences;
import edu.sdsc.mbt.*;
import edu.sdsc.sirius.dialogs.*;
import edu.sdsc.sirius.dialogs.TableImpl.*;
import edu.sdsc.sirius.io.*;
import edu.sdsc.sirius.util.Manager;
import edu.sdsc.sirius.util.DNAProperties;
import edu.sdsc.sirius.util.FileNameFilter;
import edu.sdsc.sirius.util.IOHandler;
import edu.sdsc.sirius.util.ProteinProperties;
import edu.sdsc.sirius.viewers.SequenceViewerImpl.Cell;

import javax.swing.*;
import javax.swing.border.*;
import javax.xml.namespace.QName;
import javax.xml.rpc.ParameterMode;

import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import org.apache.axis.encoding.XMLType;

import java.awt.*;
import java.awt.event.*;
import java.awt.Font;
import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import uk.ac.ebi.webservices.WSWUBlast.*;

public class BlastUniprotServicePanel extends JPanel implements SearchServicePanel {
	
	private String name = "BLAST in Uniprot";
	private Manager callable;
	
	private Entry currentEntry;
	private int currentEntryIndex = -1;//for composite entries
	
	private JPanel topPanel = new JPanel();
	private JRadioButton check1 = new JRadioButton();
	private JTextField field1 = new JTextField(100);
	private JRadioButton check2 = new JRadioButton("Fichier de squence : ");
	private JTextField field2 = new JTextField(100);
	
	private ButtonGroup group = new ButtonGroup();
	
	private JLabel label3 = new JLabel("Score max :");
	
	private JComboBox combo;
	
	private JButton search = new JButton("Rechercher");
	private JButton clear = new JButton("Effacer");
	private JButton settings = new JButton("Choix...");
	private JButton browse = new JButton("Explorer...");
	
	private JScrollPane scrollPane;

	
	private boolean protein = true;
	
	private String outputData;
	
	private JPanel panel1 = new JPanel();
	private JPanel panel2 = new JPanel();
	private JPanel panel3 = new JPanel();
	

	
	private JEditorPane editorPane = new JEditorPane();
		
	private JPanel buttonPanel = new JPanel();
	private JButton loadButton = new JButton("charger");
	private JButton alignmentButton = new JButton("Alignement");
//	private JButton printButton = new JButton("Print");
	private JButton clearButton = new JButton("Effacer les rsultats");
	private JButton outputButton = new JButton("Sortie...");
	
	private Border border1;
	private TitledBorder border2;
	private Border border;
	
	private Vector hitList = new Vector();

	private BlastResultPanel resultPanel;
	private HashMap dataToRecord = new HashMap();
	
	
	//variables for table implementation of the result list
	private Vector searchTypes;
	private DataSet set;
	private JPanel thisPanel;
	
	private volatile Thread searchRunner;
	private boolean stopSearch = false;
	
	
	public BlastUniprotServicePanel(){
		
		setLayout(new BorderLayout());

		searchTypes = new Vector();
		searchTypes.add("Nom");
		searchTypes.add("Accs");
		searchTypes.add("Description");
		searchTypes.add("Evaluer");
		
		editorPane.setEditable(false);
		
		thisPanel = this;

		topPanel.setPreferredSize(new Dimension(500, 135));
		topPanel.setLocation(new Point(0,0));
		topPanel.setBounds(0,0,500,135);
		topPanel.setLayout(new GridLayout(3,1));

		
		border1 = BorderFactory.createEmptyBorder(0,0,0,0);//left offset of the boxes in the box
		border2 = BorderFactory.createTitledBorder(new EtchedBorder(), "Rechercher les paramtres");
		border2.setTitleColor(Color.black);
		border2.setTitleFont(new Font("Dialogue", 0, 12));
		border = BorderFactory.createCompoundBorder(border1, border2);
		topPanel.setBorder(border);
		
		check1.setForeground(Color.black);
		check1.setFont(new Font("Dialogue", Font.PLAIN, 12));
		check1.setText("Cliquer la recherche dans un afficheur :");
		check1.setPreferredSize(new Dimension(150,20));
		check1.setBounds(20,25,150,20);
		check1.setLocation(new Point(20,25));
		check1.setVisible(true);
		check1.setSelected(true);
		check1.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				field2.setEditable(false);
				field2.setEnabled(false);
				field1.setEditable(true);
				field1.setEnabled(true);
			}
		});
		
		Vector types = new Vector();
		types.add("Protine");
		types.add("ADN");
		
		JComboBox typeCombo = new JComboBox(types);
		typeCombo.setPreferredSize(new Dimension(60,20));
		typeCombo.setBounds(10,0,60,20);
		typeCombo.setLocation(new Point(10,0));
		typeCombo.setVisible(true);
		
		JPanel temp1 = new JPanel();
		temp1.setLayout(null);
		temp1.setPreferredSize(new Dimension(70,20));
		temp1.add(typeCombo);

		field1.setPreferredSize(new Dimension(180,20));
		field1.setBounds(280,25,180,20);
		field1.setLocation(new Point(280,25));
		field1.setEditable(true);
		field1.setEnabled(true);
		field1.setVisible(true);
		field1.addKeyListener(new KeyAdapter(){
			public void keyPressed(KeyEvent k){
				if (k.getKeyCode() == KeyEvent.VK_ENTER){
					search.setEnabled(false);
					clear.setText("Stop");
					Thread runner = new Thread(){
						public void run(){
							process();
							clear.setText("Effacer");
							search.setEnabled(true);
						}
					};
					runner.start();
					
				}
			}
		});

		panel1.setLayout(new BorderLayout());
		panel1.add(check1, BorderLayout.WEST);
		panel1.add(field1, BorderLayout.CENTER);
		panel1.add(temp1, BorderLayout.EAST);
		panel1.setBorder(new EmptyBorder(7,7,7,7));

		
		check2.setForeground(Color.black);
		check2.setFont(new Font("Dialogue", Font.PLAIN, 12));
		check2.setPreferredSize(new Dimension(105,20));
		check2.setBounds(20,55,105,20);
		check2.setLocation(new Point(20,55));
		check2.setVisible(true);
		check2.setSelected(false);
		check2.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				field1.setEnabled(false);
				field1.setEditable(false);
				field2.setEnabled(true);
				field2.setEditable(true);
			}
		});
		
		field2.setPreferredSize(new Dimension(230,20));
		field2.setBounds(135,55,230,20);
		field2.setLocation(new Point(135,55));
		field2.setEditable(false);
		field2.setEnabled(false);
		field2.setVisible(true);
		field2.addKeyListener(new KeyAdapter(){
			public void keyPressed(KeyEvent k){
				if (k.getKeyCode() == KeyEvent.VK_ENTER){
					search.setEnabled(false);
					clear.setText("Stop");
					Thread runner = new Thread(){
						public void run(){
							process();
							clear.setText("Effacer");
							search.setEnabled(true);
						}
					};
					runner.start();
					
				}
			}
		});
		

		browse.setPreferredSize(new Dimension(90,20));
//		browse.setLocation(new Point(380, 55));
		browse.setBounds(10, 0, 90, 20);
		browse.setVisible(true);
		browse.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				check1.setSelected(false);
				field1.setEditable(false);
				field1.setEnabled(false);
				check2.setSelected(true);
				field2.setEditable(true);
				field2.setEnabled(true);
				
				
				JFileChooser chooser = new JFileChooser(StylesPreferences.workingDirectory);
				chooser.setApproveButtonText("OK");
				chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
				chooser.addChoosableFileFilter( new FileNameFilter("fst", "Format fasta") );
				chooser.addChoosableFileFilter( new FileNameFilter("aln", "Format aln") );
			
				int result = chooser.showOpenDialog(callable.getApplicationFrame());
				if (result == JFileChooser.CANCEL_OPTION){
					return;
				}
				File filename = chooser.getSelectedFile();
				field2.setText(filename.toString());
			}
		});

		panel2.setLayout(new BorderLayout());
		panel2.add(check2, BorderLayout.WEST);
		panel2.add(field2, BorderLayout.CENTER);
		
		JPanel temp = new JPanel();
		temp.setLayout(null);
		temp.setPreferredSize(new Dimension(100,20));
		temp.add(browse);
		panel2.add(temp, BorderLayout.EAST);
		panel2.setBorder(new EmptyBorder(7,7,7,7));
		
		
		label3.setForeground(Color.black);
		label3.setFont(new Font("Dialogue", Font.PLAIN, 12));
		label3.setPreferredSize(new Dimension(60,20));
		label3.setBounds(0,5,60,20);
		label3.setLocation(new Point(0,5));
		label3.setVisible(true);
		
		Vector number = new Vector();
		number.add("10");
		number.add("30");
		number.add("50");
		number.add("100");
		number.add("250");
		number.add("500");
		number.add("1000");
		
		//combo box with number of results
		combo = new JComboBox(number);
		combo.setPreferredSize(new Dimension(50,20));
		combo.setBounds(65,5,50,20);
		combo.setLocation(new Point(65,5));
		combo.setVisible(true);

		
		search.setPreferredSize(new Dimension(70,25));
		search.setLocation(new Point(0, 0));
		search.setBounds(0, 0, 70, 25);
		search.setVisible(true);
		search.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				search.setEnabled(false);
				clear.setText("Stop");
				settings.setEnabled(false);
				try{
					searchRunner = new Thread(){
						public void run(){
							process();
							clear.setText("Effacer");
							search.setEnabled(true);
							settings.setEnabled(true);
						}
					};
					searchRunner.start();
				}
				catch (Exception e){}
			}
		});
		
		clear.setPreferredSize(new Dimension(60,25));
		clear.setLocation(new Point(80, 0));
		clear.setBounds(80, 0, 60, 25);
		clear.setVisible(true);
		clear.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				if (clear.getText().equals("Stop")){
					//search is in progress, cancel it
					
					if (searchRunner.isAlive()){
						stopSearch = true;
						
						clear.setText("Effacer");
						search.setEnabled(true);
						settings.setEnabled(true);
					}
					return;
				}
				
				
				field1.setText("");
				field2.setText("");
				combo.setSelectedIndex(0);
			}
		});
		
		settings.setPreferredSize(new Dimension(90,25));
		settings.setLocation(new Point(150, 0));
		settings.setBounds(150, 0, 90, 25);
		settings.setVisible(true);
		settings.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				//check current settings
				//get display dialog status first
				boolean status = callable.getDisplayDialogStatus();
				if (status) return;
				Thread runner = new Thread(){
					public void run(){
						BlastSettingsDialog d = new BlastSettingsDialog(callable.getApplicationFrame(), callable);
					}
				};
				runner.start();
			}
		});
		
		group.add(check1);
		group.add(check2);
		
		JPanel panel3a = new JPanel();
		panel3a.setLayout(null);
		panel3a.setPreferredSize(new Dimension(250, 30));
		panel3a.add(search);
		panel3a.add(clear);
		panel3a.add(settings);
		
		JPanel panel3b = new JPanel();
		panel3b.setLayout(null);
		panel3b.setPreferredSize(new Dimension(115, 30));
		panel3b.add(label3);
		panel3b.add(combo);
		
		panel3.setLayout(new BorderLayout());
		panel3.add(panel3a, BorderLayout.WEST);
		panel3.add(panel3b, BorderLayout.EAST);
		panel3.setBorder(new EmptyBorder(3,3,3,7));
		
		
		topPanel.add(panel1);
		topPanel.add(panel2);
		topPanel.add(panel3);
		
//		editorPane.setEditable(false);
		scrollPane = new JScrollPane(editorPane);
		
		buttonPanel.setPreferredSize(new Dimension(500, 45));
//		buttonPanel.setLocation(new Point(0,0));
//		buttonPanel.setBounds(0,0,500,100);
		buttonPanel.setLayout(null);
		
		loadButton.setPreferredSize(new Dimension(60,25));
		loadButton.setLocation(new Point(10, 10));
		loadButton.setBounds(10, 10, 60, 25);
		loadButton.setVisible(true);
		loadButton.setEnabled(false);
		loadButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				Thread runner = new Thread(){
					public void run(){
						load();
					}
				};
				runner.start();
			}
		});
		
		alignmentButton.setPreferredSize(new Dimension(90,25));
		alignmentButton.setLocation(new Point(80, 10));
		alignmentButton.setBounds(80, 10, 90, 25);
		alignmentButton.setVisible(true);
		alignmentButton.setEnabled(false);
		alignmentButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				if (alignmentButton.getText().equals("Retour")){
					//it's showing the abstract, go back
					thisPanel.remove(scrollPane);
					thisPanel.add(resultPanel, BorderLayout.CENTER);
					revalidate();
					alignmentButton.setText("Alignement");
					editorPane.setText("");
					repaint();
					clearButton.setEnabled(true);
					outputButton.setEnabled(true);
				}
				else{
					viewAlignment();
				}
			}
		});
		
/*		printButton.setPreferredSize(new Dimension(100,25));
		printButton.setLocation(new Point(260, 10));
		printButton.setBounds(260, 10, 100, 25);
		printButton.setVisible(true);
		printButton.setEnabled(false);
		printButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				System.out.println("Imprimer");
			}
		});
*/
		clearButton.setPreferredSize(new Dimension(100,25));
		clearButton.setLocation(new Point(180, 10));
		clearButton.setBounds(180, 10, 100, 25);
		clearButton.setVisible(true);
		clearButton.setEnabled(false);
		clearButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
		        thisPanel.remove(resultPanel);
		        resultPanel = null;
		        outputData = null;
		        set = null;
		        thisPanel.add(scrollPane, BorderLayout.CENTER);

		        revalidate();
		        repaint();
				loadButton.setEnabled(false);
				alignmentButton.setEnabled(false);
//				printButton.setEnabled(false);
				clearButton.setEnabled(false);
				outputButton.setEnabled(false);
			}
		});

		outputButton.setPreferredSize(new Dimension(90,25));
		outputButton.setLocation(new Point(290, 10));
		outputButton.setBounds(290, 10, 90, 25);
		outputButton.setVisible(true);
		outputButton.setEnabled(false);
		outputButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent ae){
				if (outputData != null && outputData.length() > 0){
					//save it to a file
					IOHandler.saveTextFile(outputData, callable, null);
				}

			}
		});

		
		buttonPanel.add(loadButton);
		buttonPanel.add(alignmentButton);
		buttonPanel.add(clearButton);
		buttonPanel.add(outputButton);
		
		this.add(topPanel, BorderLayout.NORTH);
		this.add(scrollPane, BorderLayout.CENTER);
		this.add(buttonPanel, BorderLayout.SOUTH);
		
	}
	
	private void process(){
		
		try{
			
			//get the sequence
			String sequence = null;
			
			InputParams params = new InputParams();
			if (check1.isSelected()){
				if (currentEntry == null){
					callable.displayErrorMessage("Rien n\'a t slectionn : pas de squence de recherche");
					return;
				}
				
				if (currentEntry instanceof StructureEntry){
//					System.out.println("currentEntry");
					//derive a sequence from the structure
					StringBuffer buffer = new StringBuffer();
					Structure structure = ((StructureEntry)currentEntry).getStructure();
					StructureMap map = structure.getStructureMap();
					for (int i = 0; i < map.getResidueCount(); i++){
						Residue r = map.getResidue(i);
						if (ProteinProperties.isAminoAcid(r.getCompoundCode())){
							buffer.append(AminoAcidInfo.getLetterFromCode(r.getCompoundCode()));
						}
						else if (DNAProperties.isNucleicAcid(r.getCompoundCode())){
							buffer.append(r.getCompoundCode());//it's already single letter
						}
					}
					
					sequence = buffer.toString();
				}
				else if (currentEntry instanceof SequenceEntry){
					//get the sequence directly
					String temp = null;
					if (currentEntryIndex == -1){
						temp = ((SequenceEntry)currentEntry).getSequence().getSequence();
					}
					else{
						temp = ((SequenceEntry)currentEntry).getSequence(currentEntryIndex).getSequence();
					}
					
					Pattern pattern = Pattern.compile("-");
					Matcher matcher = pattern.matcher(temp);
					String temp2 = matcher.replaceAll("");

					Pattern pattern2 = Pattern.compile("\\*");
					Matcher matcher2 = pattern.matcher(temp2);
					sequence = matcher.replaceAll("");
				}
			}
			else{
				//file option
				
				if (field2.getText().length() == 0){
					callable.displayErrorMessage("Rien n\'a t slectionn : pas de squence de recherche");
					return;
				}
				
				String filename = field2.getText();
				Entry e = null;
				if (filename.endsWith("aln")){
					e = (new AlnEntryLoader()).load(filename, -1);
				}
				else if (filename.endsWith("fst") || filename.endsWith("fasta")){
					e = (new FastaEntryLoader()).load(filename, -1);
				}
				
				if (e == null){
					callable.displayErrorMessage("Impossible de charger le fichier slectionn. Ressayer.");
					return;
				}
				
				sequence = ((SequenceEntry)e).getSequence(0).getSequence();

			}
			
//			System.out.println("sequence = " + sequence);
			
			if (sequence == null || sequence.length() == 0){
				callable.displayErrorMessage("La squence de recherche est vide. Vrifier la slection de recherche et ressayer.");
				return;
			}
			
			//clean up the previous stuff
			if (resultPanel != null){
		        thisPanel.remove(resultPanel);
		        thisPanel.add(scrollPane, BorderLayout.CENTER);
		        resultPanel = null;
		        
		        loadButton.setEnabled(false);
		        alignmentButton.setEnabled(false);
		        clearButton.setEnabled(false);
		        outputButton.setEnabled(false);
		        
		        
		        revalidate();
		        repaint();
			}

        	hitList.clear();
        	dataToRecord.clear();
        	set = null;
			
			
			
			if (protein){
				params.setProgram("blastp");
			}
			else{
				params.setProgram("blastn");
			}
			
			String limit = (String)combo.getSelectedItem();
			int scores = 50;
			try{
				scores = Integer.parseInt(limit);
			}
			catch (NumberFormatException e){}
			
			params.setDatabase("uniprot");
			params.setMatrix(BlastSettings.matrix);
			params.setExp(BlastSettings.exp);
			params.setScores(scores);
			params.setNumal(scores);
			params.setTopcombon(BlastSettings.topcombon);
			params.setSensitivity(BlastSettings.sensitivity);
			params.setFilter(BlastSettings.filter);
			params.setEchofilter(true);
			params.setSort(BlastSettings.sort);
			params.setStats(BlastSettings.stats);
			params.setOutformat("txt");
			params.setAsync(false);
			params.setEmail("obuzko@sdsc.edu");
			params.setStrand("");
			
			Data[] inputs = new Data[1];
			Data input= new Data();
			input.setType("squence");
			input.setContent(sequence);
			inputs[0]=input;
			
		    WSWUBlastService service =  new WSWUBlastServiceLocator();
		    WSWUBlast wublast = service.getWSWUBlast();
		    String jobid = wublast.runWUBlast(params,inputs);
		    
		    if (stopSearch){
		    	stopSearch = false;
		    	return;
		    }
	
		    String[] results = getResults(jobid);
		    
		    if (stopSearch){
		    	stopSearch = false;
		    	return;
		    }
		    
		    //each item is the complete result for one blast search
		    //just take the first element
		    String output = results[0];
		    if (output == null || output.length() == 0){
		    	callable.displayErrorMessage("La recherche BLAST n\'a pas donn de rsultat.");
		    	return;
		    }
		    
		    outputData = output;
		    //parse the output
		    
//		    System.out.println(output);
		    String content = null;
		    String header = null;
		    String alignments = null;
		    int index = output.indexOf("UNIPROT");
		    if ( index != -1){
		    	content = output.substring(index);
		    }
		    else{
		    	callable.displayErrorMessage("La recherche BLAST n\'a pas donn de rsultat.");
		    	return;
		    }
		    
		    int i = content.indexOf(">");
		    if (i != -1){
		    	header = content.substring(0, i-1);
		    	alignments = content.substring(i);
		    }
		    else{
		    	callable.displayErrorMessage("La recherche BLAST n\'a pas donn de rsultat.");
		    	return;
		    }
		    
		    set = new DataSet();
		    
		    //parse header
		    int count = 1;
		    StringTokenizer tok = new StringTokenizer(header, "\n", false);
		    while (tok.hasMoreTokens()){
		    	String token = tok.nextToken();
//		    	System.out.println(token);
		    	if (token.startsWith("UNIPROT")){
		    		
		    		BlastResultRecord record = new BlastResultRecord();
					DataBlock block = new DataBlock();
		    		block.id = new Integer(count).toString();
		    		String line = token.substring(token.indexOf(":")+1);
		    		int index1 = line.indexOf(" ");
		    		
		    		String name = line.substring(0, index1);

		    		String temp = line.substring(index1+1);
		    		int index2 = temp.indexOf(" ");
		    		String temp2 = temp.substring(0, index2);

		    		String accession = temp2.trim();
		    		
		    		String temp3 = temp.substring(index2);

		    		int end = temp3.length() - 18;
		    		String description = temp3.substring(0, end).trim();
		    		String e = temp3.substring(end+6, temp3.lastIndexOf(" ")).trim();
		    		
		    		block.setName(name);
		    		record.name = name;
		    		
		    		block.setAccession(accession);
		    		record.accession = accession;
		    		
		    		block.setDescription(description);
		    		record.description = description;
		    		
		    		block.setEvalue(e);
		    		record.evalue = e;
		    		
		    		dataToRecord.put(block, record);
		    		
		    		hitList.add(record);
		    		set.addData(block);
		    		
		    		count++;
		    	}
		    }
		    
//		    System.out.println("hitList = " + hitList.size());
		    
		    //process alignments
		    StringTokenizer t = new StringTokenizer(alignments, ">", false);
		    int counter = 0;
		    while (t.hasMoreTokens()){
		    	String tt = t.nextToken();
		    	if (tt.indexOf("WARNING") != -1){
		    		((BlastResultRecord)hitList.get(counter)).alignment = tt.substring(0, tt.indexOf("WARNING"));
		    	}
		    	else if (tt.indexOf("Parameters") != -1){
		    		((BlastResultRecord)hitList.get(counter)).alignment = tt.substring(0, tt.indexOf("Parameters"));
		    	}
		    	else{
		    		((BlastResultRecord)hitList.get(counter)).alignment = tt;
		    	}
		    	counter++;
		    }
		    
		    

	        resultPanel = new BlastResultPanel(this, set);
	        this.remove(scrollPane);
	        this.add(resultPanel, BorderLayout.CENTER);

	        revalidate();
	        repaint();
	        
	        if (hitList.size() > 0){
	        	loadButton.setEnabled(true);
	        	alignmentButton.setEnabled(true);
//	        	printButton.setEnabled(true);
	        	clearButton.setEnabled(true);
	        	outputButton.setEnabled(true);
	        }
	        
		}
		catch (Exception ex){
			callable.displayExceptionMessage("Impossible de lancer la recherche BLAST", ex);
			search.setEnabled(true);
			settings.setEnabled(true);
			clear.setText("Effacer");
        	loadButton.setEnabled(false);
        	alignmentButton.setEnabled(false);
        	clearButton.setEnabled(false);
        	outputButton.setEnabled(false);
        	
        	hitList.clear();
		}
	}
	
	public String getSearchServiceName(){
		return name;
	}
	
	public void setParentApplication(Manager c){
		callable = c;
	}
	
	/**
	 * A temporary solution for now: index denotes a specific sequence within a composite entry
	 * @param entry
	 * @param index
	 */
	public void processClick(StructureComponent component, Entry entry, int index){
		//get the name from the entry
		
		/**
		 * TODO later add a more complicated analysis, to look for descriptive lines in a parsed pdb file, etc.
		 */
		this.currentEntry = entry;
		this.currentEntryIndex = index;
		if (entry == null){
			field1.setText("");
		}
		else{
			if (entry instanceof SequenceEntry){
				if (index != -1){
					Sequence s = ((SequenceEntry)entry).getSequence(index);
					field1.setText(s.getName());
				}
				else{
					Sequence s = ((SequenceEntry)entry).getSequence();
					field1.setText(s.getName());
				}
			}
			else{
				field1.setText(entry.getName());
			}
			
		}
	}
	
	public void setFocus(){
		field1.requestFocus();
	}
	
	private static String[] getResults(String jobid) throws IOException {
		try {
			
			WSWUBlastService service =  new WSWUBlastServiceLocator();
			WSWUBlast wublast = service.getWSWUBlast();
			WSFile[] results = wublast.getResults(jobid);
			
			String[] out = new String[results.length];

			for (int i=0;i<results.length;i++){
				WSFile file = results[i];
				byte[] resultbytes = wublast.poll(jobid,file.getType());
				String result = new String(resultbytes);
				out[i] = result;
			}
			
			return out;
		}
		catch (Exception e) {
			System.out.println("ERREUR :\n" + e.toString());
			e.printStackTrace();
		}
		
		return null;
	}
	
	
	private void viewAlignment(){
		if (resultPanel != null){
			
			if (resultPanel.getSelectedRowCount() == 0){
				callable.displayMessage("Rien n\'est slectionn. Choisir un rsultat et ressayer");
				return;
			}
			
			int index = resultPanel.getSelectedRows()[0];
			DataBlock block = resultPanel.getDataBlock(index);
			if (block == null) return;
			
			BlastResultRecord record = (BlastResultRecord)dataToRecord.get(block);
			try{
				editorPane.setContentType("text/html");
				StringBuffer buffer = new StringBuffer();
				buffer.append("<pre><font size=\"3\" face=\"Courier New, Courier, mono\">");
				buffer.append((String)record.alignment);
				buffer.append("</font></pre>");
				editorPane.setText(buffer.toString());
				
		        thisPanel.remove(resultPanel);
				thisPanel.add(scrollPane, BorderLayout.CENTER);
				scrollPane.getViewport().setViewPosition(new Point(0,0));
				scrollPane.revalidate();
				scrollPane.repaint();
				
				alignmentButton.setText("Retour");
				clearButton.setEnabled(false);
				outputButton.setEnabled(false);
			}
			catch (Exception ex){
				callable.displayMessage("L\'alignement n\'est pas disponible.");
				return;
			}
			
		}
	}
	
	private void load(){
		if (resultPanel != null){
			
			if (resultPanel.getSelectedRowCount() == 0){
				callable.displayMessage("Rien n\'est slectionn. Choisir un rsultat et ressayer");
				return;
			}
			
			int[] indices = resultPanel.getSelectedRows();
			
			try{
				Call call = (Call) new Service().createCall();
			    call.setTargetEndpointAddress (new java.net.URL ("http://www.ebi.ac.uk/ws/services/Dbfetch"));
			    call.setOperationName (new QName("urn:Dbfetch", "fetchData"));
			    call.addParameter( "query", XMLType.XSD_STRING, ParameterMode.IN);
			    call.addParameter( "format", XMLType.XSD_STRING, ParameterMode.IN);
			    call.addParameter( "style", XMLType.XSD_STRING, ParameterMode.IN);
			    call.setReturnType(XMLType.SOAP_ARRAY);

                //get the sequences for selected items
				for (int i = 0; i < indices.length; i++){
					if (indices[i] >= hitList.size()){
						callable.displayMessage("Aucune squence ne correspond  l\'index slectionn.");
						continue;
					}
					
					DataBlock block = resultPanel.getDataBlock(indices[i]);
					BlastResultRecord record = (BlastResultRecord)dataToRecord.get(block);
					
					String accession = record.accession;
					
					String par = "uniprot:" + accession;
					
					//get the sequence off the Uniprot server
					String[] result = null;
					try{
						result = (String[])call.invoke(new Object[] {par, "default", "raw" } );
						
						if (result.length == 0){
							callable.displayErrorMessage("Impossible de charger des squences d\'une base de donnes en ligne");
							return;
						}
						else {
							SequenceEntry entry = null;
							String name = null;
							boolean sequenceStart = false;
							StringBuffer buffer = new StringBuffer();
							for (int count = 0; count < result.length; count++){
//								System.out.println(result[count]);
								if (result[count].startsWith("ID")){
									//this is a new record, reset flags
									entry = new SequenceEntry();
									entry.setDatabaseSource("uniprot");
									entry.setDatabaseID(accession);
									sequenceStart = false;
									buffer = new StringBuffer();
									
									int index1 = result[count].indexOf(" ");
									String temp = result[count].substring(index1).trim();
									int index2 = temp.indexOf(" ");

									name = temp.substring(0, index2);
									if (name == null || name.length() == 0){
										name = accession;
									}
									entry.setName(name);
									
									
									continue;
								}
								
								//parse out sequence information
								else if (result[count].startsWith("DE")){
									String description = result[count].substring(2);
									entry.setDescription(description.trim());
								}
								else if (result[count].startsWith("OS")){
									String organism = result[count].substring(2).trim();
									//take only the first two words to avoid problems
									//with trailing common names
									int space = organism.indexOf(" ");
									String line = organism.substring(space+1);
									int n = line.indexOf(" ");
									StringBuffer out = new StringBuffer();
									out.append(organism.substring(0, space));
									out.append(" ");
									out.append(line.substring(0, n).trim());
									entry.setOrganism(out.toString());
								}
								
								if (result[count].startsWith("SQ")){
									sequenceStart = true;
									continue;
								}
								
								if (result[count].startsWith("//")){
									//time to send the entry
									String sequenceString = buffer.toString();
									Sequence sss = new Sequence(name, sequenceString);
									
									//process out to create an array of Cells
									int n = 1;
									Cell[] cells = new Cell[sequenceString.length()];
									for (int j = 0; j < sequenceString.length(); j++){
										String s = sequenceString.substring(j, j+1);
										cells[j] = new Cell(sss, j, s);
										if (!s.equals("-")){
											cells[j].residueId = n;
											n++;
										}
									}
									
									sss.setCells(cells);

									entry.setSequence(sss);
									callable.addEntry(entry, true, false);
									continue;
								}

								if (sequenceStart){
									Pattern pattern = Pattern.compile(" ");
									Matcher matcher = pattern.matcher(result[count]);
									String compressed = matcher.replaceAll("");
									buffer.append(compressed);
								}
							}
						}
					}
					catch (Exception e){
						
//						e.printStackTrace();
					}
				}
			}
			catch (Exception ex){
				callable.displayMessage("l\'alignement n\'est pas disponible pour cette ligne.");
				return;
			}
			
		}
	}

	public final Entry getCurrentEntry() {
		return currentEntry;
	}

	public final void setCurrentEntry(Entry currentEntry) {
		this.currentEntry = currentEntry;
		field1.setText("");
	}

	
	
	
}
